@@ -53,78 +53,93 @@
 #include "qrunner.h"
 
 #include "winkeyer.h"
+#include "nanoIO.h"
 
 using namespace std;
 
 #define XMT_FILT_LEN 256
 #define QSK_DELAY_LEN 4*XMT_FILT_LEN
+#define CW_FFT_SIZE 2048 // must be a factor of 2
+
+static double nano_d2d = 0;
+static int nano_wpm = 0;
 
 const cw::SOM_TABLE cw::som_table[] = {
 	/* Prosigns */
-	{'=',	"<BT>",   {1.0,  0.33,  0.33,  0.33, 1.0,   0, 0}	}, // 0
-	{'~',	"<AA>",   { 0.33, 1.0,  0.33, 1.0,   0,   0, 0}	}, // 1
-	{'%',	"<AS>",   { 0.33, 1.0,  0.33,  0.33,  0.33,   0, 0} 	}, // 2
-	{'+',	"<AR>",   { 0.33, 1.0,  0.33, 1.0,  0.33,   0, 0} 	}, // 3
-	{'>',	"<SK>",   { 0.33,  0.33,  0.33, 1.0,  0.33, 1.0, 0}	}, // 4
-	{'<',	"<KN>",   {1.0,  0.33, 1.0, 1.0,  0.33,   0, 0} 	}, // 5
-	{'&',	"<INT>",  { 0.33,  0.33, 1.0,  0.33, 1.0,   0, 0}	}, // 6
-	{'}',	"<HM>",   { 0.33,  0.33,  0.33,  0.33, 1.0, 1.0, 0}	}, // 7
-	{'{',	"<VE>",   { 0.33,  0.33,  0.33, 1.0,  0.33,   0, 0}	}, // 8
+	{"-...-",	{1.0,  0.33,  0.33,  0.33, 1.0,   0, 0} },
+	{".-.-",	{ 0.33, 1.0,  0.33, 1.0,   0,   0, 0} },
+	{".-...",	{ 0.33, 1.0,  0.33,  0.33,  0.33,   0, 0} },
+	{".-.-.",	{ 0.33, 1.0,  0.33, 1.0,  0.33,   0, 0} },
+	{"...-.-",	{ 0.33,  0.33,  0.33, 1.0,  0.33, 1.0, 0} },
+	{"-.--.",	{1.0,  0.33, 1.0, 1.0,  0.33,   0, 0} },
+	{"..-.-",	{ 0.33,  0.33, 1.0,  0.33, 1.0,   0, 0} },
+	{"....--",	{ 0.33,  0.33,  0.33,  0.33, 1.0, 1.0, 0} },
+	{"...-.",	{ 0.33,  0.33,  0.33, 1.0,  0.33,   0, 0} },
 	/* ASCII 7bit letters */
-	{'A',	"A",	{ 0.33, 1.0,   0,   0,   0,   0, 0}	},
-	{'B',	"B",	{1.0,  0.33,  0.33,  0.33,   0,   0, 0}	},
-	{'C',	"C",	{1.0,  0.33, 1.0,  0.33,   0,   0, 0}	},
-	{'D',	"D",	{1.0,  0.33,  0.33,   0,   0,   0, 0} 	},
-	{'E',	"E",	{ 0.33,   0,   0,   0,   0,   0, 0}	},
-	{'F',	"F",	{ 0.33,  0.33, 1.0,  0.33,   0,   0, 0}	},
-	{'G',	"G",	{1.0, 1.0,  0.33,   0,   0,   0, 0}	},
-	{'H',	"H",	{ 0.33,  0.33,  0.33,  0.33,   0,   0, 0}	},
-	{'I',	"I",	{ 0.33,  0.33,   0,   0,   0,   0, 0}	},
-	{'J',	"J",	{ 0.33, 1.0, 1.0, 1.0,   0,   0, 0}	},
-	{'K',	"K",	{1.0,  0.33, 1.0,   0,   0,   0, 0}	},
-	{'L',	"L",	{ 0.33, 1.0,  0.33,  0.33,   0,   0, 0}	},
-	{'M',	"M",	{1.0, 1.0,   0,   0,   0,   0, 0}	},
-	{'N',	"N",	{1.0,  0.33,   0,   0,   0,   0, 0}	},
-	{'O',	"O",	{1.0, 1.0, 1.0,   0,   0,   0, 0}	},
-	{'P',	"P",	{ 0.33, 1.0, 1.0,  0.33,   0,   0, 0}	},
-	{'Q',	"Q",	{1.0, 1.0,  0.33, 1.0,   0,   0, 0}	},
-	{'R',	"R",	{ 0.33, 1.0,  0.33,   0,   0,   0, 0}	},
-	{'S',	"S",	{ 0.33,  0.33,  0.33,   0,   0,   0, 0}	},
-	{'T',	"T",	{1.0,   0,   0,   0,   0,   0, 0}	},
-	{'U',	"U",	{ 0.33,  0.33, 1.0,   0,   0,   0, 0}	},
-	{'V',	"V",	{ 0.33,  0.33,  0.33, 1.0,   0,   0, 0}	},
-	{'W',	"W",	{ 0.33, 1.0, 1.0,   0,   0,   0, 0}	},
-	{'X',	"X",	{1.0,  0.33,  0.33, 1.0,   0,   0, 0}	},
-	{'Y',	"Y",	{1.0,  0.33, 1.0, 1.0,   0,   0, 0}	},
-	{'Z',	"Z",	{1.0, 1.0,  0.33,  0.33,   0,   0, 0}	},
+	{".-",		{ 0.33, 1.0,   0,   0,   0,   0, 0}	},
+	{"-...",	{1.0,  0.33,  0.33,  0.33,   0,   0, 0}	},
+	{"-.-.",	{1.0,  0.33, 1.0,  0.33,   0,   0, 0}	},
+	{"-..",		{1.0,  0.33,  0.33,   0,   0,   0, 0} 	},
+	{".",		{ 0.33,   0,   0,   0,   0,   0, 0}	},
+	{"..-.",	{ 0.33,  0.33, 1.0,  0.33,   0,   0, 0}	},
+	{"--.",		{1.0, 1.0,  0.33,   0,   0,   0, 0}	},
+	{"....",	{ 0.33,  0.33,  0.33,  0.33,   0,   0, 0}	},
+	{"..",		{ 0.33,  0.33,   0,   0,   0,   0, 0}	},
+	{".---J",	{ 0.33, 1.0, 1.0, 1.0,   0,   0, 0}	},
+	{"-.-",		{1.0,  0.33, 1.0,   0,   0,   0, 0}	},
+	{".-..",	{ 0.33, 1.0,  0.33,  0.33,   0,   0, 0}	},
+	{"--",		{1.0, 1.0,   0,   0,   0,   0, 0}	},
+	{"-.",		{1.0,  0.33,   0,   0,   0,   0, 0}	},
+	{"---",		{1.0, 1.0, 1.0,   0,   0,   0, 0}	},
+	{".--.",	{ 0.33, 1.0, 1.0,  0.33,   0,   0, 0}	},
+	{"--.-",	{1.0, 1.0,  0.33, 1.0,   0,   0, 0}	},
+	{".-.",		{ 0.33, 1.0,  0.33,   0,   0,   0, 0}	},
+	{"...",		{ 0.33,  0.33,  0.33,   0,   0,   0, 0}	},
+	{"-",		{1.0,   0,   0,   0,   0,   0, 0}	},
+	{"..-",		{ 0.33,  0.33, 1.0,   0,   0,   0, 0}	},
+	{"...-",	{ 0.33,  0.33,  0.33, 1.0,   0,   0, 0}	},
+	{".--",		{ 0.33, 1.0, 1.0,   0,   0,   0, 0}	},
+	{"-..-",	{1.0,  0.33,  0.33, 1.0,   0,   0, 0}	},
+	{"-.--",	{1.0,  0.33, 1.0, 1.0,   0,   0, 0}	},
+	{"--..",	{1.0, 1.0,  0.33,  0.33,   0,   0, 0}	},
 	/* Numerals */
-	{'0',	"0",	{1.0, 1.0, 1.0, 1.0, 1.0,   0, 0}	},
-	{'1',	"1",	{ 0.33, 1.0, 1.0, 1.0, 1.0,   0, 0}	},
-	{'2',	"2",	{ 0.33,  0.33, 1.0, 1.0, 1.0,   0, 0}	},
-	{'3',	"3",	{ 0.33,  0.33,  0.33, 1.0, 1.0,   0, 0}	},
-	{'4',	"4",	{ 0.33,  0.33,  0.33,  0.33, 1.0,   0, 0}	},
-	{'5',	"5",	{ 0.33,  0.33,  0.33,  0.33,  0.33,   0, 0}	},
-	{'6',	"6",	{1.0,  0.33,  0.33,  0.33,  0.33,   0, 0}	},
-	{'7',	"7",	{1.0, 1.0,  0.33,  0.33,  0.33,   0, 0}	},
-	{'8',	"8",	{1.0, 1.0, 1.0,  0.33,  0.33,   0, 0}	},
-	{'9',	"9",	{1.0, 1.0, 1.0, 1.0,  0.33,   0, 0}	},
+	{"-----",	{1.0, 1.0, 1.0, 1.0, 1.0,   0, 0}	},
+	{".----",	{ 0.33, 1.0, 1.0, 1.0, 1.0,   0, 0}	},
+	{"..---",	{ 0.33,  0.33, 1.0, 1.0, 1.0,   0, 0}	},
+	{"...--",	{ 0.33,  0.33,  0.33, 1.0, 1.0,   0, 0}	},
+	{"....-",	{ 0.33,  0.33,  0.33,  0.33, 1.0,   0, 0}	},
+	{".....",	{ 0.33,  0.33,  0.33,  0.33,  0.33,   0, 0}	},
+	{"-....",	{1.0,  0.33,  0.33,  0.33,  0.33,   0, 0}	},
+	{"--...",	{1.0, 1.0,  0.33,  0.33,  0.33,   0, 0}	},
+	{"---..",	{1.0, 1.0, 1.0,  0.33,  0.33,   0, 0}	},
+	{"----.",	{1.0, 1.0, 1.0, 1.0,  0.33,   0, 0}	},
 	/* Punctuation */
-	{'\\',	"\\",	{ 0.33, 1.0,  0.33,  0.33, 1.0,  0.33, 0}	},
-	{'\'',	"'",	{ 0.33, 1.0, 1.0, 1.0, 1.0,  0.33, 0}	},
-	{'$',	"$",	{ 0.33,  0.33,  0.33, 1.0,  0.33,  0.33,1.0}	},
-	{'(',	"(",	{1.0,  0.33, 1.0, 1.0,  0.33,   0, 0}	},
-	{')',	")",	{1.0,  0.33, 1.0, 1.0,  0.33, 1.0, 0}	},
-	{',',	",",	{1.0, 1.0,  0.33,  0.33, 1.0, 1.0, 0}	},
-	{'-',	"-",	{1.0,  0.33,  0.33,  0.33,  0.33, 1.0, 0}	},
-	{'.',	".",	{ 0.33, 1.0,  0.33, 1.0,  0.33, 1.0, 0}	},
-	{'/',	"/",	{1.0,  0.33,  0.33, 1.0,  0.33,   0, 0}	},
-	{':',	":",	{1.0, 1.0, 1.0,  0.33,  0.33,  0.33, 0}	},
-	{';',	";",	{1.0,  0.33, 1.0,  0.33, 1.0,  0.33, 0}	},
-	{'?',	"?",	{ 0.33,  0.33, 1.0, 1.0,  0.33,  0.33, 0}	},
-	{'_',	"_",	{ 0.33,  0.33, 1.0, 1.0,  0.33, 1.0, 0}	},
-	{'@',	"@",	{ 0.33, 1.0, 1.0,  0.33, 1.0,  0.33, 0}	},
-	{'!',	"!",	{1.0,  0.33, 1.0,  0.33, 1.0, 1.0, 0}	},
-	{0, NULL, {0.0}}
+	{".-..-.",	{ 0.33, 1.0,  0.33,  0.33, 1.0,  0.33, 0}	},
+	{".----.",	{ 0.33, 1.0, 1.0, 1.0, 1.0,  0.33, 0}	},
+	{"...-..-",	{ 0.33,  0.33,  0.33, 1.0,  0.33,  0.33, 1.0}	},
+	{"-.---.",	{1.0,  0.33, 1.0, 1.0,  0.33,   0, 0}	},
+	{"-.--.-",	{1.0,  0.33, 1.0, 1.0,  0.33, 1.0, 0}	},
+	{"--..--",	{1.0, 1.0,  0.33,  0.33, 1.0, 1.0, 0}	},
+	{"-....-",	{1.0,  0.33,  0.33,  0.33,  0.33, 1.0, 0}	},
+	{".-.-.-",	{ 0.33, 1.0,  0.33, 1.0,  0.33, 1.0, 0}	},
+	{"-..-.",	{1.0,  0.33,  0.33, 1.0,  0.33,   0, 0}	},
+	{"---...",	{1.0, 1.0, 1.0,  0.33,  0.33,  0.33, 0}	},
+	{"-.-.-.",	{1.0,  0.33, 1.0,  0.33, 1.0,  0.33, 0}	},
+	{"..--..",	{ 0.33,  0.33, 1.0, 1.0,  0.33,  0.33, 0}	},
+	{"..--.-",	{ 0.33,  0.33, 1.0, 1.0,  0.33, 1.0, 0}	},
+	{".--.-.",	{ 0.33, 1.0, 1.0,  0.33, 1.0,  0.33, 0}	},
+	{"-.-.--",	{1.0,  0.33, 1.0,  0.33, 1.0, 1.0, 0}	},
+
+	{".-.-",	{0.33, 1.0, 0.33, 1.0, 0, 0 , 0}  },	// A umlaut, A aelig
+	{".--.-",	{0.33, 1.0, 1.0, 0.33, 1.0, 0, 0 }  },	// A ring
+	{"-.-..",	{1.0, 0.33, 1.0, 0.33, 0.33, 0, 0} },	// C cedilla
+	{".-..-",	{0.33, 1.0, 0.33, 0.33, 1.0, 0, 0} },	// E grave
+	{"..-..",	{0.33, 0.33, 1.0, 0.33, 0.33, 0, 0} },	// E acute
+	{"---.",	{1.0, 1.0, 1.0, 0.33, 0, 0, 0} },		// O acute, O umlat, O slash
+	{"--.--",	{1.0, 1.0, 0.33, 1.0, 1.0, 0, 0} },		// N tilde
+	{"..--",	{0.33, 0.33, 1.0, 1.0, 0, 0, 0} },		// U umlaut, U circ
+
+	{"", {0.0}}
 };
 
 int cw::normalize(float *v, int n, int twodots)
@@ -152,38 +167,39 @@ int cw::normalize(float *v, int n, int twodots)
 }
 
 
-const char * cw::find_winner (float *inbuf, int twodots)
+std::string cw::find_winner (float *inbuf, int twodots)
 {
 	float diffsf = 999999999999.0;
 
-	if ( normalize (inbuf, WGT_SIZE, twodots) == 0) return NULL;
+	if ( normalize (inbuf, WGT_SIZE, twodots) == 0) return " ";
 
-	const SOM_TABLE * winner = NULL;
-	for ( const SOM_TABLE * som = som_table; som->chr != 0; som++) {
+	int winner = -1;
+	for ( int n = 0; som_table[n].rpr.length(); n++) {
 		 /* Compute the distance between codebook and input entry */
 		float difference = 0.0;
 	   	for (int i = 0; i < WGT_SIZE; i++) {
-			float diff = (inbuf[i] - som->wgt[i]);
+			float diff = (inbuf[i] - som_table[n].wgt[i]);
 					difference += diff * diff;
 					if (difference > diffsf) break;
 	  		}
 
 	 /* If distance is smaller than previous distances */
 			if (difference < diffsf) {
-	  			winner = som;
+	  			winner = n;
 	  			diffsf = difference;
 			}
 	}
 
-	if (winner != NULL)
-		return winner->prt;
-	else
-		return NULL;
+	if (!som_table[winner].rpr.empty()) {
+		string sc = morse.rx_lookup(som_table[winner].rpr);
+		if (sc.empty()) return "*";
+		else return sc;
+	} else
+		return "*";
 }
 
-void cw::tx_init(SoundBase *sc)
+void cw::tx_init()
 {
-	scard = sc;
 	phaseacc = 0;
 	lastsym = 0;
 	qskphase = 0;
@@ -210,6 +226,8 @@ void cw::rx_init()
 	update_Status();
 	usedefaultWPM = false;
 	scope_clear = true;
+
+	viewcw.restart();
 }
 
 void cw::init()
@@ -229,7 +247,7 @@ void cw::init()
 		set_freq(wf->Carrier());
 
 	trackingfilter->reset();
-	cw_adaptive_receive_threshold = (long int)trackingfilter->run(2 * cw_send_dot_length);
+	two_dots = (long int)trackingfilter->run(2 * cw_send_dot_length);
 	put_cwRcvWPM(cw_send_speed);
 	for (int i = 0; i < OUTBUFSIZE; i++)
 		outbuf[i] = qskbuf[i] = 0.0;
@@ -238,11 +256,12 @@ void cw::init()
 	prosigns = progdefaults.CW_prosigns;
 	stopflag = false;
 	maxval = 0;
+
+	if (use_nanoIO) set_nanoCW();
+
 }
 
 cw::~cw() {
-	if (hilbert) delete hilbert;
-	if (cw_FIR_filter) delete cw_FIR_filter;
 	if (cw_FFT_filter) delete cw_FFT_filter;
 	if (cw_xmt_filter) delete cw_xmt_filter;
 	if (bitfilter) delete bitfilter;
@@ -264,7 +283,7 @@ cw::cw() : modem()
 	cw_ptr = 0;
 	clrcount = CLRCOUNT;
 
-	samplerate = CWSampleRate;
+	samplerate = CW_SAMPLERATE;
 	fragmentsize = CWMaxSymLen;
 
 	cw_speed  = progdefaults.CWspeed;
@@ -272,14 +291,14 @@ cw::cw() : modem()
 
 	cw_send_speed = cw_speed;
 	cw_receive_speed = cw_speed;
-	cw_adaptive_receive_threshold = 2 * DOT_MAGIC / cw_speed;
-	cw_noise_spike_threshold = cw_adaptive_receive_threshold / 4;
-	cw_send_dot_length = DOT_MAGIC / cw_send_speed;
+	two_dots = 2 * KWPM / cw_speed;
+	cw_noise_spike_threshold = two_dots / 4;
+	cw_send_dot_length = KWPM / cw_send_speed;
 	cw_send_dash_length = 3 * cw_send_dot_length;
 	symbollen = (int)round(samplerate * 1.2 / progdefaults.CWspeed);
 	fsymlen = (int)round(samplerate * 1.2 / progdefaults.CWfarnsworth);
 
-	memset(rx_rep_buf, 0, sizeof(rx_rep_buf));
+	rx_rep_buf.clear();
 
 // block of variables that get updated each time speed changes
 	pipesize = (22 * samplerate * 12) / (progdefaults.CWspeed * 160);
@@ -289,9 +308,7 @@ cw::cw() : modem()
 	cwTrack = true;
 	phaseacc = 0.0;
 	FFTphase = 0.0;
-	FIRphase = 0.0;
 	FFTvalue = 0.0;
-	FIRvalue = 0.0;
 	pipeptr = 0;
 	clrcount = 0;
 
@@ -302,23 +319,13 @@ cw::cw() : modem()
 	agc_peak = 1.0;
 	in_replay = 0;
 
-	use_fft_filter = progdefaults.CWuse_fft_filter;
 	use_matched_filter = progdefaults.CWmfilt;
 
 	bandwidth = progdefaults.CWbandwidth;
 	if (use_matched_filter)
 		progdefaults.CWbandwidth = bandwidth = 2.0 * progdefaults.CWspeed / 1.2;
 
-	hilbert = new C_FIR_filter(); // hilbert transform used by FFT filter
-	hilbert->init_hilbert(37, 1);
-
-	cw_FIR_filter = new C_FIR_filter();
-	cw_FIR_filter->init_lowpass (CW_FIRLEN, DEC_RATIO, progdefaults.CWspeed/(1.2 * samplerate));
-
-//overlap and add filter length should be a factor of 2
-// low pass implementation
-	FilterFFTLen = 4096;
-	cw_FFT_filter = new fftfilt(progdefaults.CWspeed/(1.2 * samplerate), FilterFFTLen);
+	cw_FFT_filter = new fftfilt(progdefaults.CWspeed/(1.2 * samplerate), CW_FFT_SIZE);
 
 // transmit filtering
 
@@ -336,13 +343,17 @@ cw::cw() : modem()
 	for (int i = 0; i < QSK_DELAY_LEN; i++) qsk_signal[i] = 0.0;
 	cw_xmt_filter = new fftfilt( lwr, upr, 2*XMT_FILT_LEN);// transmit filter length
 
-	int bfv = symbollen / 50;
+	int bfv = symbollen / 32;
 	if (bfv < 1) bfv = 1;
 	bitfilter = new Cmovavg(bfv);
 
 	trackingfilter = new Cmovavg(TRACKING_FILTER_SIZE);
 
 	makeshape();
+
+	nano_wpm = progdefaults.CWspeed;
+	nano_d2d = progdefaults.CWdash2dot;
+
 	sync_parameters();
 	REQ(static_cast<void (waterfall::*)(int)>(&waterfall::Bandwidth), wf, (int)bandwidth);
 	REQ(static_cast<int (Fl_Value_Slider2::*)(double)>(&Fl_Value_Slider2::value), sldrCWbandwidth, (int)bandwidth);
@@ -357,12 +368,10 @@ cw::cw() : modem()
 // SHOULD ONLY BE CALLED FROM THE rx_processing loop
 void cw::reset_rx_filter()
 {
-	if (use_fft_filter != progdefaults.CWuse_fft_filter ||
-		use_matched_filter != progdefaults.CWmfilt ||
+	if (use_matched_filter != progdefaults.CWmfilt ||
 		cw_speed != progdefaults.CWspeed ||
 		(bandwidth != progdefaults.CWbandwidth && !use_matched_filter)) {
 
-		use_fft_filter = progdefaults.CWuse_fft_filter;
 		use_matched_filter = progdefaults.CWmfilt;
 		cw_send_speed = cw_speed = progdefaults.CWspeed;
 
@@ -372,13 +381,9 @@ void cw::reset_rx_filter()
 			bandwidth = progdefaults.CWbandwidth;
 
 		double fbw = 0.5 * bandwidth / samplerate;
-		if (use_fft_filter) { // FFT filter
-			cw_FFT_filter->create_lpf(fbw);
-			FFTphase = 0;
-		} else { // FIR filter
-			cw_FIR_filter->init_lowpass (CW_FIRLEN, DEC_RATIO, fbw);
-			FIRphase = 0;
-		}
+		cw_FFT_filter->create_lpf(fbw);
+		FFTphase = 0;
+
 		REQ(static_cast<void (waterfall::*)(int)>(&waterfall::Bandwidth),
 			wf, (int)bandwidth);
 		REQ(static_cast<int (Fl_Value_Slider2::*)(double)>(&Fl_Value_Slider2::value),
@@ -388,26 +393,23 @@ void cw::reset_rx_filter()
 		if (pipesize < 0) pipesize = 512;
 		if (pipesize > MAX_PIPE_SIZE) pipesize = MAX_PIPE_SIZE;
 
-		cw_adaptive_receive_threshold = 2 * DOT_MAGIC / cw_speed;
-		cw_noise_spike_threshold = cw_adaptive_receive_threshold / 4;
-		cw_send_dot_length = DOT_MAGIC / cw_send_speed;
+		two_dots = 2 * KWPM / cw_speed;
+		cw_noise_spike_threshold = two_dots / 4;
+		cw_send_dot_length = KWPM / cw_send_speed;
 		cw_send_dash_length = 3 * cw_send_dot_length;
 		symbollen = (int)round(samplerate * 1.2 / progdefaults.CWspeed);
 		fsymlen = (int)round(samplerate * 1.2 / progdefaults.CWfarnsworth);
 
 		phaseacc = 0.0;
 		FFTphase = 0.0;
-		FIRphase = 0.0;
 		FFTvalue = 0.0;
-		FIRvalue = 0.0;
 		pipeptr = 0;
 		clrcount = 0;
 		smpl_ctr = 0;
 
-		memset(rx_rep_buf, 0, sizeof(rx_rep_buf));
+		rx_rep_buf.clear();
 
-
-	int bfv = symbollen / 50;
+	int bfv = symbollen / 32;
 	if (bfv < 1) bfv = 1;
 	bitfilter->setLength(bfv);
 
@@ -426,7 +428,7 @@ void cw::sync_transmit_parameters()
 	wpm = usedefaultWPM ? progdefaults.defCWspeed : progdefaults.CWspeed;
 	fwpm = progdefaults.CWfarnsworth;
 
-	cw_send_dot_length = DOT_MAGIC / progdefaults.CWspeed;
+	cw_send_dot_length = KWPM / progdefaults.CWspeed;
 	cw_send_dash_length = 3 * cw_send_dot_length;
 
 	nusymbollen = (int)round(samplerate * 1.2 / wpm);
@@ -446,13 +448,24 @@ void cw::sync_transmit_parameters()
 
 void cw::sync_parameters()
 {
+	if (use_nanoIO) {
+		if (nano_wpm != progdefaults.CWspeed) {
+			nano_wpm = progdefaults.CWspeed;
+			set_nanoWPM(progdefaults.CWspeed);
+		}
+		if (nano_d2d != progdefaults.CWdash2dot) {
+			nano_d2d = progdefaults.CWdash2dot;
+			set_nano_dash2dot(progdefaults.CWdash2dot);
+		}
+	}
+
 	sync_transmit_parameters();
 
 // check if user changed the tracking or the cw default speed
 	if ((cwTrack != progdefaults.CWtrack) ||
 		(cw_send_speed != progdefaults.CWspeed)) {
 		trackingfilter->reset();
-		cw_adaptive_receive_threshold = 2 * cw_send_dot_length;
+		two_dots = 2 * cw_send_dot_length;
 		put_cwRcvWPM(cw_send_speed);
 	}
 	cwTrack = progdefaults.CWtrack;
@@ -465,20 +478,20 @@ void cw::sync_parameters()
 		lowerwpm = progdefaults.CWlowerlimit;
 	if (upperwpm > progdefaults.CWupperlimit)
 		upperwpm = progdefaults.CWupperlimit;
-	cw_lower_limit = 2 * DOT_MAGIC / upperwpm;
-	cw_upper_limit = 2 * DOT_MAGIC / lowerwpm;
+	cw_lower_limit = 2 * KWPM / upperwpm;
+	cw_upper_limit = 2 * KWPM / lowerwpm;
 
 	if (cwTrack)
-		cw_receive_speed = DOT_MAGIC / (cw_adaptive_receive_threshold / 2);
+		cw_receive_speed = KWPM / (two_dots / 2);
 	else {
 		cw_receive_speed = cw_send_speed;
-		cw_adaptive_receive_threshold = 2 * cw_send_dot_length;
+		two_dots = 2 * cw_send_dot_length;
 	}
 
 	if (cw_receive_speed > 0)
-		cw_receive_dot_length = DOT_MAGIC / cw_receive_speed;
+		cw_receive_dot_length = KWPM / cw_receive_speed;
 	else
-		cw_receive_dot_length = DOT_MAGIC / 5;
+		cw_receive_dot_length = KWPM / 5;
 
 	cw_receive_dash_length = 3 * cw_receive_dot_length;
 
@@ -489,28 +502,20 @@ void cw::sync_parameters()
 
 //=======================================================================
 // cw_update_tracking()
-// This gets called everytime we have a dot dash sequence or a dash dot
-// sequence. Since we have semi validated tone durations, we can try and
-// track the cw speed by adjusting the cw_adaptive_receive_threshold variable.
-// This is done with moving average filters for both dot & dash.
 //=======================================================================
 
-void cw::update_tracking(int idot, int idash)
+inline void cw::update_tracking(int dur_1, int dur_2)
 {
-	int dot, dash;
-	if (idot > cw_lower_limit && idot < cw_upper_limit)
-		dot = idot;
-	else
-		dot = cw_send_dot_length;
-	if (idash > cw_lower_limit && idash < cw_upper_limit)
-		dash = idash;
-	else
-		dash = cw_send_dash_length;
+static int min_dot = KWPM / 200;
+static int max_dash = 3 * KWPM / 5;
+	if ((dur_1 > dur_2) && (dur_1 > 4 * dur_2)) return;
+	if ((dur_2 > dur_1) && (dur_2 > 4 * dur_1)) return;
+	if (dur_1 < min_dot || dur_2 < min_dot) return;
+	if (dur_2 > max_dash || dur_2 > max_dash) return;
 
-	cw_adaptive_receive_threshold = (long int)trackingfilter->run((dash + dot) / 2);
+	two_dots = trackingfilter->run((dur_1 + dur_2) / 2);
 
-//	if (!use_matched_filter)
-		sync_parameters();
+	sync_parameters();
 }
 
 //=======================================================================
@@ -577,22 +582,34 @@ cmplx cw::mixer(cmplx in)
 
 void cw::decode_stream(double value)
 {
-	const char *c, *somc;
-	char *cptr;
+	std::string sc;
+	std::string somc;
+	int attack = 0;
+	int decay = 0;
+	switch (progdefaults.cwrx_attack) {
+		case 0: attack = 100; break;
+		case 1: default: attack = 50; break;
+		case 2: attack = 25;
+	}
+	switch (progdefaults.cwrx_decay) {
+		case 0: decay = 1000; break;
+		case 1: default : decay = 500; break;
+		case 2: decay = 250;
+	}
 
-	sig_avg = decayavg(sig_avg, value, 1000);
+	sig_avg = decayavg(sig_avg, value, decay);
 
 	if (value < sig_avg) {
 		if (value < noise_floor)
-			noise_floor = decayavg(noise_floor, value, 100);
+			noise_floor = decayavg(noise_floor, value, attack);
 		else 
-			noise_floor = decayavg(noise_floor, value, 1000);
+			noise_floor = decayavg(noise_floor, value, decay);
 	}
 	if (value > sig_avg)  {
 		if (value > agc_peak)
-			agc_peak = decayavg(agc_peak, value, 100);
+			agc_peak = decayavg(agc_peak, value, attack);
 		else
-			agc_peak = decayavg(agc_peak, value, 1000); 
+			agc_peak = decayavg(agc_peak, value, decay); 
 	}
 
 	float norm_noise  = noise_floor / agc_peak;
@@ -604,8 +621,9 @@ void cw::decode_stream(double value)
 	else
 		value = 0;
 
-	if (norm_noise) metric = clamp(100 * sig_avg / norm_noise , 0, 100);
-	else metric = 0;
+	metric = 0.8 * metric;
+	if ((noise_floor > 1e-4) && (noise_floor < sig_avg))
+		metric += 0.2 * clamp(2.5 * (20*log10(sig_avg / noise_floor)) , 0, 100);
 
 	float diff = (norm_sig - norm_noise);
 
@@ -619,33 +637,31 @@ void cw::decode_stream(double value)
 // Power detection using hysterisis detector
 // upward trend means tone starting
 		if ((value > progdefaults.CWupper) && (cw_receive_state != RS_IN_TONE)) {
-			handle_event(CW_KEYDOWN_EVENT, NULL);
+			handle_event(CW_KEYDOWN_EVENT, sc);
 		}
 // downward trend means tone stopping
 		if ((value < progdefaults.CWlower) && (cw_receive_state == RS_IN_TONE)) {
-			handle_event(CW_KEYUP_EVENT, NULL);
+			handle_event(CW_KEYUP_EVENT, sc);
 		}
 	}
 
-	if (handle_event(CW_QUERY_EVENT, &c) == CW_SUCCESS) {
+	if (handle_event(CW_QUERY_EVENT, sc) == CW_SUCCESS) {
 		update_syncscope();
 		synchscope = 100;
 		if (progdefaults.CWuseSOMdecoding) {
-			somc = find_winner(cw_buffer, cw_adaptive_receive_threshold);
-			cptr = (char*)somc;
-			if (somc != NULL) {
-				while (*cptr != '\0')
-					put_rx_char(progdefaults.rx_lowercase ? tolower(*cptr++) : *cptr++,FTextBase::CTRL);
-			}
-			if (strlen(c) == 1 && *c == ' ')
-				put_rx_char(progdefaults.rx_lowercase ? tolower(*c) : *c);
+			somc = find_winner(cw_buffer, two_dots);
+			if (!somc.empty())
+				for (size_t n = 0; n < somc.length(); n++)
+					put_rx_char(
+						somc[n],
+						somc[0] == '<' ? FTextBase::CTRL : FTextBase::RECV);
 			cw_ptr = 0;
 			memset(cw_buffer, 0, sizeof(cw_buffer));
 		} else {
-			if (strlen(c) == 1)
-				put_rx_char(progdefaults.rx_lowercase ? tolower(*c) : *c);
-			else while (*c)
-				put_rx_char(progdefaults.rx_lowercase ? tolower(*c++) : *c++, FTextBase::CTRL);
+			for (size_t n = 0; n < sc.length(); n++)
+				put_rx_char(
+					sc[n],
+					sc[0] == '<' ? FTextBase::CTRL : FTextBase::RECV);
 		}
 	} else if (--synchscope == 0) {
 		synchscope = 25;
@@ -688,46 +704,27 @@ void cw::rx_FFTprocess(const double *buf, int len)
 	} //while (len-- > 0)
 }
 
-void cw::rx_FIRprocess(const double *buf, int len)
-{
-	cmplx z;
-
-	while (len-- > 0) {
-		z = cmplx ( *buf * cos(FIRphase), *buf * sin(FIRphase) );
-		buf++;
-
-		FIRphase += TWOPI * frequency / samplerate;
-		if (FIRphase > TWOPI) FIRphase -= TWOPI;
-
-		if (cw_FIR_filter->run ( z, z )) {
-
-// update the basic sample counter used for morse timing
-			smpl_ctr += DEC_RATIO;
-// demodulate
-			FIRvalue = abs(z);
-			FIRvalue = bitfilter->run(FIRvalue);
-
-			decode_stream(FIRvalue);
-		}
-	}
-}
-
 static bool cwprocessing = false;
 
 int cw::rx_process(const double *buf, int len)
 {
-	if (cwprocessing) return 0;
+	if (cwprocessing)
+		return 0;
+
 	cwprocessing = true;
+
 	reset_rx_filter();
 
-	if (use_fft_filter)
-		rx_FFTprocess(buf, len);
-	else
-		rx_FIRprocess(buf, len);
+	rx_FFTprocess(buf, len);
 
 	if (!clrcount--) clear_syncscope();
 
 	display_metric(metric);
+
+	if ( (dlgViewer->visible() || progStatus.show_channels ) 
+		&& !bHighSpeed && !bHistory )
+		viewcw.rx_process(buf, len);
+
 	cwprocessing = false;
 
 	return 0;
@@ -737,15 +734,9 @@ int cw::rx_process(const double *buf, int len)
 
 // Compare two timestamps, and return the difference between them in usecs.
 
-int cw::usec_diff(unsigned int earlier, unsigned int later)
+inline int cw::usec_diff(unsigned int earlier, unsigned int later)
 {
-// Compare the timestamps.
-// At 4 WPM, the dash length is 3*(1200000/4)=900,000 usecs, and
-// the word gap is 2,100,000 usecs.
-	if (earlier >= later) {
-		return 0;
-	} else
-		return (int) (((double) (later - earlier) * USECS_PER_SEC) / samplerate);
+	return (earlier >= later) ? 0 : (later - earlier);
 }
 
 
@@ -760,7 +751,7 @@ int cw::usec_diff(unsigned int earlier, unsigned int later)
 //	If there is no data ready, CW_ERROR is returned.
 //=======================================================================
 
-int cw::handle_event(int cw_event, const char **c)
+int cw::handle_event(int cw_event, string &sc)
 {
 	static int space_sent = true;	// for word space logic
 	static int last_element = 0;	// length of last dot/dash
@@ -774,7 +765,7 @@ int cw::handle_event(int cw_event, const char **c)
 		cw_ptr = 0;
 		memset(cw_buffer, 0, sizeof(cw_buffer));
 		smpl_ctr = 0;					// reset audio sample counter
-		memset(rx_rep_buf, 0, sizeof(rx_rep_buf));
+		rx_rep_buf.clear();
 		break;
 	case CW_KEYDOWN_EVENT:
 // A receive tone start can only happen while we
@@ -784,7 +775,7 @@ int cw::handle_event(int cw_event, const char **c)
 // first tone in idle state reset audio sample counter
 		if (cw_receive_state == RS_IDLE) {
 			smpl_ctr = 0;
-			memset(rx_rep_buf, 0, sizeof(rx_rep_buf));
+			rx_rep_buf.clear();
 			cw_rr_current = 0;
 			cw_ptr = 0;
 		}
@@ -836,18 +827,18 @@ int cw::handle_event(int cw_event, const char **c)
 		last_element = element_usec;
 // ok... do we have a dit or a dah?
 // a dot is anything shorter than 2 dot times
-		if (element_usec <= cw_adaptive_receive_threshold) {
-			rx_rep_buf[cw_rr_current++] = CW_DOT_REPRESENTATION;
+		if (element_usec <= two_dots) {
+			rx_rep_buf += CW_DOT_REPRESENTATION;
 	//		printf("%d dit ", last_element/1000);  // print dot length
 			cw_buffer[cw_ptr++] = (float)last_element;
 		} else {
 // a dash is anything longer than 2 dot times
-			rx_rep_buf[cw_rr_current++] = CW_DASH_REPRESENTATION;
+			rx_rep_buf += CW_DASH_REPRESENTATION;
 			cw_buffer[cw_ptr++] = (float)last_element;
 		}
 // We just added a representation to the receive buffer.
 // If it's full, then reset everything as it probably noise
-		if (cw_rr_current == RECEIVE_CAPACITY - 1) {
+		if (rx_rep_buf.length() > MAX_MORSE_ELEMENTS) {
 			cw_receive_state = RS_IDLE;
 			cw_rr_current = 0;	// reset decoding pointer
 			cw_ptr = 0;
@@ -855,7 +846,7 @@ int cw::handle_event(int cw_event, const char **c)
 			return CW_ERROR;
 		} else {
 // zero terminate representation
-			rx_rep_buf[cw_rr_current] = 0;
+//			rx_rep_buf.clear();
 			cw_buffer[cw_ptr] = 0.0;
 		}
 // All is well.  Move to the more normal after-tone state.
@@ -868,15 +859,6 @@ int cw::handle_event(int cw_event, const char **c)
 // CW_SUCCESS is returned when there is a printable character. Nothing to do if we are in a tone
 		if (cw_receive_state == RS_IN_TONE)
 			return CW_ERROR;
-// in this call we expect a pointer to a char to be valid
-		if (c == NULL) {
-// else we had no place to put character...
-			cw_receive_state = RS_IDLE;
-			cw_rr_current = 0;
-			cw_ptr = 0;
-// reset decoding pointer
-			return CW_ERROR;
-		}
 // compute length of silence so far
 		sync_parameters();
 		element_usec = usec_diff(cw_rr_end_timestamp, smpl_ctr);
@@ -890,13 +872,12 @@ int cw::handle_event(int cw_event, const char **c)
 			element_usec <= (4 * cw_receive_dot_length) &&
 			cw_receive_state == RS_AFTER_TONE) {
 // Look up the representation
-//cout << "CW_QUERY medium time after keyup: " << rx_rep_buf;
-			*c = morse.rx_lookup(rx_rep_buf);
-//cout <<": " << *c <<flush;
-			if (*c == NULL) {
+			sc = morse.rx_lookup(rx_rep_buf);
+			if (sc.empty()) {
 // invalid decode... let user see error
-				*c = "*";
+				sc = "*";
 			}
+			rx_rep_buf.clear();
 			cw_receive_state = RS_IDLE;
 			cw_rr_current = 0;	// reset decoding pointer
 			space_sent = false;
@@ -907,7 +888,7 @@ int cw::handle_event(int cw_event, const char **c)
 // LONG time since keyup... check for a word space
 // FARNSWOTH MOD HERE -->
 		if ((element_usec > (4 * cw_receive_dot_length)) && !space_sent) {
-			*c = " ";
+			sc = " ";
 			space_sent = true;
 			return CW_SUCCESS;
 		}
@@ -925,11 +906,11 @@ int cw::handle_event(int cw_event, const char **c)
 // this is 1/2 cycle of a raised cosine
 //===========================================================================
 
-double keyshape[KNUM];
+double keyshape[CWKNUM];
 
 void cw::makeshape()
 {
-	for (int i = 0; i < KNUM; i++) keyshape[i] = 1.0;
+	for (int i = 0; i < CWKNUM; i++) keyshape[i] = 1.0;
 
 	switch (QSKshape) {
 		case 1: // blackman
@@ -1085,10 +1066,11 @@ void cw::send_symbol(int bits, int len)
 				kpost = keydown - knum + (int)(progdefaults.CWpost * 8);
 		} else
 			kpost = keydown + (int)(progdefaults.CWpost * 8);
-			if (kpost < 0) kpost = 0;
-		} else {
-			kpre = 0;
+		if (kpost < 0)
 			kpost = 0;
+	} else {
+		kpre = 0;
+		kpost = 0;
 	}
 
 	if (firstelement) {
@@ -1213,17 +1195,26 @@ void cw::send_symbol(int bits, int len)
 
 void cw::send_ch(int ch)
 {
-	int code;
+	string code;
 	int chout = ch;
 	int flen;
 
 	sync_parameters();
+
 // handle word space separately (7 dots spacing)
 // last char already had 3 elements of inter-character spacing
 
+	double T = (50.0 - 31 * fsymlen / symbollen) / 19.0;
+
+	if (!progdefaults.CWusefarnsworth || (progdefaults.CWspeed >= progdefaults.CWfarnsworth))
+		T = 1.0;
+
+	int tc = 3.0 * T * symbollen;
+	int tw = 7.0 * T * symbollen;
+
 	if ((chout == ' ') || (chout == '\n')) {
 		firstelement = false;
-		flen = 4 * symbollen;
+		flen = tw - tc - ( T > 1.0 ? fsymlen : 0);
 		while (flen - symbollen > 0) {
 			send_symbol(0, symbollen);
 			flen -= symbollen;
@@ -1233,49 +1224,48 @@ void cw::send_ch(int ch)
 		return;
 	}
 
-// convert character code to a morse representation
-	if ((chout < 256) && (chout >= 0)) {
-		code = morse.tx_lookup(chout); //cw_tx_lookup(ch);
-		firstelement = true;
-	} else {
-		code = 0x04; 	// two extra dot spaces
-		firstelement = false;
-	}
-
-// loop sending out binary bits of cw character
-// at WPM or Farnsworth rate
 	if (progdefaults.CWusefarnsworth && (progdefaults.CWspeed <= progdefaults.CWfarnsworth))
 		flen = fsymlen;
 	else
 		flen = symbollen;
 
-	int charlen = 0;
-	while (code > 1) {
-		send_symbol(code, flen);
-		charlen++;
-		code = code >> 1;
+	code = morse.tx_lookup(chout);
+	if (!code.length()) {
+		return;
 	}
 
+	firstelement = true;
+
+//	while (code.length()) {
+	for (size_t n = 0; n < code.length(); n++) {
+		send_symbol(0, flen);
+		send_symbol(1, flen);
+		if (code[n] == '-') {
+			send_symbol( 1, flen );
+			send_symbol( 1, flen );
+		}
+	}
+	send_symbol(0, flen);
+	send_symbol(0, flen);
+	send_symbol(0, flen);
+
 // inter character space at WPM/FWPM rate
-	flen = symbollen;
-	if (progdefaults.CWusefarnsworth && (progdefaults.CWspeed <= progdefaults.CWfarnsworth))
-		flen += (symbollen - fsymlen)*charlen;
-
-	while(flen - symbollen > 0) {
-		send_symbol(0, symbollen);
-		flen -= symbollen;
+//	if (progdefaults.CWusefarnsworth && (progdefaults.CWspeed <= progdefaults.CWfarnsworth)) {
+	if (T > 1.0) {
+		flen = tc - 3 * flen;
+		while(flen - symbollen > 0) {
+			send_symbol(0, symbollen);
+			flen -= symbollen;
+		}
+		if (flen) send_symbol(0, flen);
 	}
-	if (flen) send_symbol(0, flen);
-
-	FL_AWAKE();
 
 	if (ch != -1) {
-		string prtstr = morse.tx_print(ch);
-		if (prtstr.length() == 1)
-			put_echo_char(progdefaults.rx_lowercase ? tolower(prtstr[0]) : prtstr[0]);
-		else
-			for (size_t n = 0; n < prtstr.length(); n++)
-				put_echo_char(progdefaults.rx_lowercase ? tolower(prtstr[n]) : prtstr[n], FTextBase::CTRL);
+		string prtstr = morse.tx_print();
+		for (size_t n = 0; n < prtstr.length(); n++)
+			put_echo_char(
+				prtstr[n],
+				prtstr[0] == '<' ? FTextBase::CTRL : FTextBase::XMIT);
 	}
 }
 
@@ -1298,12 +1288,46 @@ int cw::tx_process()
 
 	c = get_tx_char();
 
+	if (use_nanoIO) set_nanoCW();
+
+	if (c == GET_TX_CHAR_NODATA) {
+		if (stopflag) {
+			stopflag = false;
+			if (progdefaults.CW_bpf_on) { // flush the transmit bandpass filter
+				int n = cw_xmt_filter->flush_size();
+				for (int i = 0; i < n; i++) outbuf[i] = 0;
+				nb_filter(outbuf, outbuf, n);
+			}
+			put_echo_char('\n');
+			return -1;
+		}
+		Fl::awake();
+		MilliSleep(50);
+		return 0;
+	}
+
 	if (progStatus.WK_online) {
 		if (c == GET_TX_CHAR_ETX || stopflag) {
 			stopflag = false;
+			put_echo_char('\n');
 			return -1;
 		}
-		if (WK_send_char(c)) return -1; // WinKeyer problem
+		if (WK_send_char(c)){
+			put_echo_char('\n');
+			return -1; // WinKeyer problem
+		}
+		return 0;
+	}
+
+	if (use_nanoIO) {
+		if (c == GET_TX_CHAR_ETX || stopflag) {
+			stopflag = false;
+			put_echo_char('\n');
+			nano_serial_flush();
+			return -1;
+		}
+		nano_send_char(c);
+		put_echo_char(c);
 		return 0;
 	}
 
@@ -1314,18 +1338,14 @@ int cw::tx_process()
 			for (int i = 0; i < n; i++) outbuf[i] = 0;
 			nb_filter(outbuf, outbuf, n);
 		}
+		put_echo_char('\n');
 		return -1;
 	}
+
 	acc_symbols = 0;
 	send_ch(c);
 
-	xmt_samples = char_samples = acc_symbols;
-
-//	printf("%5s %d samples, overhead %d, %f sec's\n",
-//		ascii3[c & 0xff],
-//		char_samples,
-//		ovhd_samples,
-//		1.0 * char_samples / samplerate);
+	char_samples = acc_symbols;
 
 	return 0;
 }
