@@ -25,12 +25,14 @@
 #include <cmath>
 #include <cstring>
 #include <vector>
+#include <queue>
 #include <list>
 #include <stdlib.h>
 
 #include <FL/Fl_Text_Display.H>
 #include <FL/Fl_Text_Buffer.H>
 
+#include "config.h"
 #include "fl_digi.h"
 #include "rigsupport.h"
 #include "modem.h"
@@ -53,6 +55,10 @@
 
 #include "confdialog.h"
 
+#ifdef __MINGW32__
+#  include "compat/mingw.h"
+#endif
+
 LOG_FILE_SOURCE(debug::LOG_FD);
 
 //#define DXC_DEBUG 1
@@ -78,6 +84,7 @@ pthread_mutex_t DXcluster_mutex     = PTHREAD_MUTEX_INITIALIZER;
 Socket *DXcluster_socket = 0;
 
 enum DXC_STATES {DISCONNECTED, CONNECTING, CONNECTED};
+
 int  DXcluster_state = DISCONNECTED;
 bool DXcluster_exit = false;
 bool DXcluster_enabled = false;
@@ -206,10 +213,74 @@ void show_error(string buff)
 #endif
 }
 
-#include <queue>
+static void odd_even()
+{
+	char hdr[100];
+	snprintf(hdr, sizeof(hdr), 
+			"@F%d@S%d@.Spotter    Freq     Dx Station       Notes                      UTC    LOC",
+			progdefaults.DXC_textfont,
+			progdefaults.DXC_textsize);
+	reports_header->clear();
+	reports_header->add(hdr);
+	reports_header->redraw();
+
+	snprintf(odd, sizeof(odd), "@B%u@C%u@F%d@S%d@.",
+			progdefaults.DXC_odd_color,
+			FL_BLACK,
+			progdefaults.DXC_textfont,
+			progdefaults.DXC_textsize);
+
+	snprintf(even, sizeof(even), "@B%u@C%u@F%d@S%d@.",
+		progdefaults.DXC_even_color,
+		FL_BLACK,
+		progdefaults.DXC_textfont,
+		progdefaults.DXC_textsize);
+
+}
+
+void dxc_lines_redraw()
+{
+	guard_lock dxcc_lock(&dxc_line_mutex);
+
+	odd_even();
+
+	int n = brws_dx_cluster->size();
+	if (n == 0) return;
+
+	queue<string> lines;
+	string dxc_line;
+	size_t p;
+	for (int i = 0; i < n; i++) {
+		dxc_line = brws_dx_cluster->text(i+1);
+		p = dxc_line.find(".");
+		if (p != string::npos) dxc_line.erase(0,p+1);
+		lines.push(dxc_line);
+	}
+	brws_dx_cluster->clear();
+
+	for (int i = 0; i < n; i++) {
+		if (i % 2)
+			dxc_line.assign(even);
+		else
+			dxc_line.assign(odd);
+		dxc_line.append(lines.front());
+		lines.pop();
+		brws_dx_cluster->insert(1, dxc_line.c_str());
+	}
+	if (progdefaults.dxc_topline)
+		brws_dx_cluster->make_visible(1);
+	else
+		brws_dx_cluster->bottomline(brws_dx_cluster->size());
+
+	brws_dx_cluster->redraw();
+}
+
 void dxc_lines()
 {
 	guard_lock dxcc_lock(&dxc_line_mutex);
+
+	odd_even();
+
 	int n = brws_dx_cluster->size();
 
 	if (n == 0) return;
@@ -609,32 +680,11 @@ void DXcluster_recv_data()
 	string tempbuff;
 	try {
 		guard_lock dxc_lock(&DXcluster_mutex);
-		DXcluster_socket->recv(tempbuff);
-		if (tempbuff.empty()) {
-			if (DXcluster_state == CONNECTING) {
-				DXcluster_connect_timeout--;
-				if (DXcluster_connect_timeout <= 0) {
-					REQ(show_error, "Connection attempt timed out");
-					DXcluster_state = DISCONNECTED;
-					set_btn_dxcc_connect(false);
-					REQ(dxc_label);
-					DXcluster_socket->shut_down();
-					DXcluster_socket->close();
-					delete DXcluster_socket;
-					DXcluster_socket = 0;
-				}
-			}
-			return;
+		if (DXcluster_state == CONNECTED) {
+			DXcluster_socket->recv(tempbuff);
+			if (!tempbuff.empty())
+				REQ(parse_DXcluster_stream, tempbuff);
 		}
-		if (DXcluster_state == CONNECTING) {
-			DXcluster_state = CONNECTED;
-			REQ(dxc_label);
-
-			LOG_INFO( "Connected to dxserver %s:%s",
-				progdefaults.dxcc_host_url.c_str(),
-				progdefaults.dxcc_host_port.c_str() );
-		}
-		REQ(parse_DXcluster_stream, tempbuff);
 	} catch (const SocketException& e) {
 		LOG_ERROR("Error %d, %s", e.error(), e.what());
 	}
@@ -717,6 +767,8 @@ void DXcluster_submit()
 void rf_af(long &rf, long &af)
 {
 	mode_t md = active_modem->get_mode();
+	string testmode = qso_opMODE->value();
+	bool xcvr_useFSK = (testmode.find("RTTY") != string::npos);
 
 	if (md == MODE_SSB) {
 		af = 0;
@@ -732,28 +784,19 @@ void rf_af(long &rf, long &af)
 		}
 	}
 	else if (md == MODE_RTTY) {
-		int shift;
-		switch (progdefaults.rtty_shift) {
-		case 0 : shift = 23; break;
-		case 1 : shift = 85; break;
-		case 2 : shift = 160; break;
-		case 3 : shift = 170; break;
-		case 4 : shift = 182; break;
-		case 5 : shift = 200; break;
-		case 6 : shift = 240; break;
-		case 7 : shift = 350; break;
-		case 8 : shift = 425; break;
-		case 9 : shift = 850; break;
-		default: shift = 0;
-		}
-		af = progdefaults.RTTYsweetspot;
-		if (progdefaults.useMARKfreq) {
-			if (wf->USB()) rf -= (af + shift / 2);
-			else           rf += (af - shift / 2);
-		}
-		else {
-			if (wf->USB()) rf -= af;
-			else           rf += af;
+		if (xcvr_useFSK) {
+			af = progdefaults.xcvr_FSK_MARK + rtty::SHIFT[progdefaults.rtty_shift]/2;
+		} else {
+			int shift = rtty::SHIFT[progdefaults.rtty_shift];
+			af = progdefaults.RTTYsweetspot;
+			if (progdefaults.useMARKfreq) {
+				if (wf->USB()) rf -= (af + shift / 2);
+				else           rf += (af - shift / 2);
+			}
+			else {
+				if (wf->USB()) rf -= af;
+				else           rf += af;
+			}
 		}
 	} else {
 		af = progdefaults.PSKsweetspot;
@@ -896,49 +939,63 @@ bool connect_to_cluster;
 
 void DXcluster_doconnect()
 {
+	int result = 0;
 	if (connect_to_cluster) {
 		REQ(clear_dxcluster_viewer);
 		try {
-			if (DXcluster_socket) {
+			if (DXcluster_state == DISCONNECTED) {
+				if (DXcluster_socket) {
+					DXcluster_socket->shut_down();
+					DXcluster_socket->close();
+					delete DXcluster_socket;
+					DXcluster_socket = 0;
+					DXcluster_state = DISCONNECTED;
+					REQ(dxc_label);
+				}
+				Address addr = Address(
+					progdefaults.dxcc_host_url.c_str(),
+					progdefaults.dxcc_host_port.c_str() );
+
+				DXcluster_socket = new Socket( addr );
+
+				DXcluster_socket->set_nonblocking(true);
+				DXcluster_socket->set_timeout((double)(DXCLUSTER_SOCKET_TIMEOUT / 1000.0));
+			}
+
+			result = DXcluster_socket->connect();
+			if ( (result == 0) || (result == EISCONN) || (result == EALREADY) ) {
+				DXcluster_state = CONNECTED;
+				REQ(dxc_label);
+				LOG_INFO( "Connected to dxserver %s:%s",
+					progdefaults.dxcc_host_url.c_str(),
+					progdefaults.dxcc_host_port.c_str() );
+				connect_changed = false;
+				return;
+			} else if ( (result == EWOULDBLOCK) || (result == EINPROGRESS) )
+				DXcluster_state = CONNECTING;
+			else
+				DXcluster_state = DISCONNECTED;
+
+			if ((DXcluster_state == CONNECTING) &&
+				(DXcluster_connect_timeout-- <= 0) ) {
+				REQ(show_error, "Connection attempt timed out");
+				DXcluster_state = DISCONNECTED;
+				set_btn_dxcc_connect(false);
+				REQ(dxc_label);
 				DXcluster_socket->shut_down();
 				DXcluster_socket->close();
 				delete DXcluster_socket;
 				DXcluster_socket = 0;
-				DXcluster_state = DISCONNECTED;
-				REQ(dxc_label);
+			} else {
+				LOG_INFO("Connecting %f seconds remaining", DXcluster_connect_timeout * DXCLUSTER_LOOP_TIME * 0.001);
 			}
-			Address addr = Address(
-				progdefaults.dxcc_host_url.c_str(),
-				progdefaults.dxcc_host_port.c_str() );
-
-			DXcluster_socket = new Socket( addr );
-
-			DXcluster_socket->set_nonblocking(true);
-			DXcluster_socket->set_timeout((double)(DXCLUSTER_SOCKET_TIMEOUT / 1000.0));
-
-			DXcluster_socket->connect();
-
-			DXcluster_state = CONNECTING;
-			REQ(dxc_label);
-			string temp = "Connecting to ";
-			temp.append(progdefaults.dxcc_host_url).append(":");
-			temp.append(progdefaults.dxcc_host_port);
-			REQ(show_error, temp);
-			DXcluster_connect_timeout =
-				(DXCLUSTER_CONNECT_TIMEOUT) / (DXCLUSTER_SOCKET_TIMEOUT + DXCLUSTER_LOOP_TIME);
+			connect_changed = false;
+			return;
 		} catch (const SocketException& e) {
 			LOG_ERROR("%s", e.what() );
-			delete DXcluster_socket;
-			DXcluster_socket = 0;
-			DXcluster_state = DISCONNECTED;
-			progStatus.cluster_connected = false;
 			REQ(show_error, e.what());
-			set_btn_dxcc_connect(false);
-			REQ(dxc_label);
-			logged_in = false;
 		}
-	}
-	else {
+	}	else {
 		if (!DXcluster_socket) return;
 
 		try {
@@ -983,8 +1040,9 @@ void *DXcluster_loop(void *args)
 	while(1) {
 		MilliSleep(DXCLUSTER_LOOP_TIME);
 		if (DXcluster_exit) break;
-		if (connect_changed) DXcluster_doconnect();
-		if (DXcluster_state != DISCONNECTED) DXcluster_recv_data();
+		if (connect_changed || (DXcluster_state == CONNECTING) )
+			DXcluster_doconnect();
+		if (DXcluster_state == CONNECTED) DXcluster_recv_data();
 	}
 	// exit the DXCC thread
 	SET_THREAD_CANCEL();
@@ -1167,7 +1225,7 @@ void dxcluster_hosts_select(Fl_Button*, void*)
 	int line_nbr = brws_dxcluster_hosts->value();
 	if (line_nbr == 0) return;
 	host_line = brws_dxcluster_hosts->text(line_nbr);
-	string	host_name, 
+	string	host_name,
 			host_port,
 			host_login,
 			host_password;
@@ -1182,7 +1240,7 @@ void dxcluster_hosts_select(Fl_Button*, void*)
 	host_port = host_line.substr(0, p);
 	host_line.erase(0, p+1);
 	p = host_line.find(":");
-	if (p == string::npos) 
+	if (p == string::npos)
 		host_login = host_line;
 	else {
 		host_login = host_line.substr(0, p);
@@ -1205,10 +1263,6 @@ void dxcluster_hosts_select(Fl_Button*, void*)
 	progdefaults.dxcc_password = host_password;
 	inp_dxcc_password->value(host_password.c_str());
 	inp_dxcc_password->redraw();
-
-std::cout << "host " << host_name << "\nport " << host_port <<
-"\nlogin " << host_login << "\npassword " << host_password << std::endl;
-
 }
 
 void dxcluster_hosts_delete(Fl_Button*, void*)
