@@ -90,12 +90,12 @@ const struct mode_info_t mode_info[NUM_MODES] = {
 	{ MODE_NAVTEX, &navtex_modem, "NAVTEX", "NAVTEX", "NAVTEX", "TOR", "NAVTEX", DISABLED_IO },
 	{ MODE_SITORB, &sitorb_modem, "SITORB", "SITORB", "SITORB", "TOR", "SITORB", DISABLED_IO },
 
-	{ MODE_MT63_500S, &mt63_500S_modem, "MT63-500S", "MT63-500S", "MT63-500S", "MT63", "MT63-500S", ARQ_IO | KISS_IO  },
-	{ MODE_MT63_500L, &mt63_500L_modem, "MT63-500L", "MT63-500L", "MT63-500L", "MT63", "MT63-500L", ARQ_IO | KISS_IO  },
-	{ MODE_MT63_1000S, &mt63_1000S_modem, "MT63-1KS", "MT63-1000S", "MT63-1XXS", "MT63", "MT63 1KS", ARQ_IO | KISS_IO  },
-	{ MODE_MT63_1000L, &mt63_1000L_modem, "MT63-1KL", "MT63-1000L", "MT63-1XXL", "MT63", "MT63 1KL", ARQ_IO | KISS_IO  },
-	{ MODE_MT63_2000S, &mt63_2000S_modem, "MT63-2KS", "MT63-2000S", "MT63-2XXS", "MT63", "MT63 2KS", ARQ_IO | KISS_IO  },
-	{ MODE_MT63_2000L, &mt63_2000L_modem, "MT63-2KL", "MT63-2000L", "MT63-2XXL", "MT63", "MT63 2KL", ARQ_IO | KISS_IO  },
+	{ MODE_MT63_500S, &mt63_500S_modem, "MT63-500S", "MT63-500S", "MT63-500S", "MT63-500S", "MT63-500S", ARQ_IO | KISS_IO  },
+	{ MODE_MT63_500L, &mt63_500L_modem, "MT63-500L", "MT63-500L", "MT63-500L", "MT63-500L", "MT63-500L", ARQ_IO | KISS_IO  },
+	{ MODE_MT63_1000S, &mt63_1000S_modem, "MT63-1KS", "MT63-1000S", "MT63-1XXS", "MT63-1KS", "MT63 1KS", ARQ_IO | KISS_IO  },
+	{ MODE_MT63_1000L, &mt63_1000L_modem, "MT63-1KL", "MT63-1000L", "MT63-1XXL", "MT63-1KL", "MT63 1KL", ARQ_IO | KISS_IO  },
+	{ MODE_MT63_2000S, &mt63_2000S_modem, "MT63-2KS", "MT63-2000S", "MT63-2XXS", "MT63-2KS", "MT63 2KS", ARQ_IO | KISS_IO  },
+	{ MODE_MT63_2000L, &mt63_2000L_modem, "MT63-2KL", "MT63-2000L", "MT63-2XXL", "MT63-2KL", "MT63 2KL", ARQ_IO | KISS_IO  },
 
 	{ MODE_PSK31, &psk31_modem, "BPSK31", "BPSK-31", "PSK31", "PSK31", "P31", ARQ_IO | KISS_IO  },
 	{ MODE_PSK63, &psk63_modem, "BPSK63", "BPSK-63", "PSK63", "PSK63", "P63", ARQ_IO | KISS_IO  },
@@ -131,36 +131,36 @@ const struct mode_info_t mode_info[NUM_MODES] = {
 	{ MODE_8PSK1200F, &_8psk1200f_modem, "8PSK1200F", "8PSK-1200F", "8PSK1200F", "8PSK1200F", "8PSK1200F", ARQ_IO | KISS_IO },
 
 	{ MODE_OLIVIA, &olivia_modem, "OLIVIA", "OLIVIA", "OLIVIA", "OLIVIA", "OL", DISABLED_IO },
-	{ MODE_OLIVIA_4_250, &olivia_4_250_modem, "Olivia-4-250", "OL 4-250", "OLIV 4/250", "OLIVIA", "OL4/250", ARQ_IO  },
-	{ MODE_OLIVIA_8_250, &olivia_8_250_modem, "Olivia-8-250", "OL 8-250", "OLIV 8/250", "OLIVIA", "OL8/250", ARQ_IO  },
-	{ MODE_OLIVIA_4_500, &olivia_4_500_modem, "Olivia-4-500", "OL 4-500", "OLIV 4/500", "OLIVIA", "OL4/500", ARQ_IO | KISS_IO },
-	{ MODE_OLIVIA_8_500, &olivia_8_500_modem, "Olivia-8-500", "OL 8-500", "OLIV 8/500", "OLIVIA", "OL8/500", ARQ_IO | KISS_IO },
-	{ MODE_OLIVIA_16_500, &olivia_16_500_modem, "Olivia-16-500", "OL 16-500", "OLIV 16/500", "OLIVIA", "OL16/500", ARQ_IO | KISS_IO },
-	{ MODE_OLIVIA_8_1000, &olivia_8_1000_modem, "Olivia-8-1K", "OL 8-1K", "OLIV 8/1K", "OLIVIA", "OL8/1K", ARQ_IO | KISS_IO  },
-	{ MODE_OLIVIA_16_1000, &olivia_16_1000_modem, "Olivia-16-1K", "OL 16-1K", "OLIV 16/1K", "OLIVIA", "OL16/1K", ARQ_IO | KISS_IO  },
-	{ MODE_OLIVIA_32_1000, &olivia_32_1000_modem, "Olivia-32-1K", "OL 32-1K", "OLIV 32/1K", "OLIVIA", "OL32/1K", ARQ_IO | KISS_IO  },
-	{ MODE_OLIVIA_64_2000, &olivia_64_2000_modem, "Olivia-64-2K", "OL 64-2K", "OLIV 64/2K", "OLIVIA", "OL64/2K", ARQ_IO | KISS_IO  },
+	{ MODE_OLIVIA_4_250, &olivia_4_250_modem, "Olivia-4-250", "OL 4-250", "OLIV 4-250", "OLIVIA-4/250", "OL4/250", ARQ_IO  },
+	{ MODE_OLIVIA_8_250, &olivia_8_250_modem, "Olivia-8-250", "OL 8-250", "OLIV 8-250", "OLIVIA-8/250", "OL8/250", ARQ_IO  },
+	{ MODE_OLIVIA_4_500, &olivia_4_500_modem, "Olivia-4-500", "OL 4-500", "OLIV 4-500", "OLIVIA-4/500", "OL4/500", ARQ_IO | KISS_IO },
+	{ MODE_OLIVIA_8_500, &olivia_8_500_modem, "Olivia-8-500", "OL 8-500", "OLIV 8-500", "OLIVIA-8/500", "OL8/500", ARQ_IO | KISS_IO },
+	{ MODE_OLIVIA_16_500, &olivia_16_500_modem, "Olivia-16-500", "OL 16-500", "OLIV 16-500", "OLIVIA-16/500", "OL16/500", ARQ_IO | KISS_IO },
+	{ MODE_OLIVIA_8_1000, &olivia_8_1000_modem, "Olivia-8-1K", "OL 8-1K", "OLIV 8-1K", "OLIVIA-8/1K", "OL8/1K", ARQ_IO | KISS_IO  },
+	{ MODE_OLIVIA_16_1000, &olivia_16_1000_modem, "Olivia-16-1K", "OL 16-1K", "OLIV 16-1K", "OLIVIA-16/1K", "OL16/1K", ARQ_IO | KISS_IO  },
+	{ MODE_OLIVIA_32_1000, &olivia_32_1000_modem, "Olivia-32-1K", "OL 32-1K", "OLIV 32-1K", "OLIVIA-32/1K", "OL32/1K", ARQ_IO | KISS_IO  },
+	{ MODE_OLIVIA_64_2000, &olivia_64_2000_modem, "Olivia-64-2K", "OL 64-2K", "OLIV 64-2K", "OLIVIA-64/2K", "OL64/2K", ARQ_IO | KISS_IO  },
 
 	{ MODE_RTTY, &rtty_modem, "RTTY", "RTTY", "RTTY", "RTTY", "RY", DISABLED_IO },
 
-	{ MODE_THORMICRO, &thormicro_modem, "THOR Micro", "THOR Micro", "THORM", "THOR", "THM", DISABLED_IO  },
-	{ MODE_THOR4, &thor4_modem, "THOR4", "THOR 4", "THOR4", "THOR", "TH4", DISABLED_IO  },
-	{ MODE_THOR5, &thor5_modem, "THOR5", "THOR 5", "THOR5", "THOR", "TH5", DISABLED_IO  },
-	{ MODE_THOR8, &thor8_modem, "THOR8", "THOR 8", "THOR8", "THOR", "TH8", DISABLED_IO  },
-	{ MODE_THOR11, &thor11_modem, "THOR11", "THOR 11", "THOR11", "THOR", "TH11", DISABLED_IO  },
-	{ MODE_THOR16, &thor16_modem, "THOR16", "THOR 16", "THOR16", "THOR", "TH16", ARQ_IO | KISS_IO },
-	{ MODE_THOR22, &thor22_modem, "THOR22", "THOR 22", "THOR22", "THOR", "TH22", ARQ_IO | KISS_IO },
-	{ MODE_THOR25x4, &thor25x4_modem, "THOR25x4", "THOR 25 x4", "THOR25x4", "THOR", "TH25", ARQ_IO | KISS_IO },
-	{ MODE_THOR50x1, &thor50x1_modem, "THOR50x1", "THOR 50 x1", "THOR50x1", "THOR", "TH51", ARQ_IO | KISS_IO },
-	{ MODE_THOR50x2, &thor50x2_modem, "THOR50x2", "THOR 50 x2", "THOR50x2", "THOR", "TH52", ARQ_IO | KISS_IO },
-	{ MODE_THOR100, &thor100_modem, "THOR100", "THOR 100", "THOR100", "THOR", "TH10", ARQ_IO | KISS_IO },
+	{ MODE_THORMICRO, &thormicro_modem, "THOR Micro", "THOR Micro", "THORM", "THOR-M", "THM", DISABLED_IO  },
+	{ MODE_THOR4, &thor4_modem, "THOR4", "THOR 4", "THOR4", "THOR4", "TH4", DISABLED_IO  },
+	{ MODE_THOR5, &thor5_modem, "THOR5", "THOR 5", "THOR5", "THOR5", "TH5", DISABLED_IO  },
+	{ MODE_THOR8, &thor8_modem, "THOR8", "THOR 8", "THOR8", "THOR8", "TH8", DISABLED_IO  },
+	{ MODE_THOR11, &thor11_modem, "THOR11", "THOR 11", "THOR11", "THOR11", "TH11", DISABLED_IO  },
+	{ MODE_THOR16, &thor16_modem, "THOR16", "THOR 16", "THOR16", "THOR16", "TH16", ARQ_IO | KISS_IO },
+	{ MODE_THOR22, &thor22_modem, "THOR22", "THOR 22", "THOR22", "THOR22", "TH22", ARQ_IO | KISS_IO },
+	{ MODE_THOR25x4, &thor25x4_modem, "THOR25x4", "THOR 25 x4", "THOR25x4", "THOR-25X4", "TH25", ARQ_IO | KISS_IO },
+	{ MODE_THOR50x1, &thor50x1_modem, "THOR50x1", "THOR 50 x1", "THOR50x1", "THOR-50X1", "TH51", ARQ_IO | KISS_IO },
+	{ MODE_THOR50x2, &thor50x2_modem, "THOR50x2", "THOR 50 x2", "THOR50x2", "THOR-50X2", "TH52", ARQ_IO | KISS_IO },
+	{ MODE_THOR100, &thor100_modem, "THOR100", "THOR 100", "THOR100", "THOR-100", "TH100", ARQ_IO | KISS_IO },
 
-	{ MODE_THROB1, &throb1_modem, "THROB1", "Throb 1", "", "THRB", "TB1", DISABLED_IO },
-	{ MODE_THROB2, &throb2_modem, "THROB2", "Throb 2", "", "THRB", "TB2", DISABLED_IO },
-	{ MODE_THROB4, &throb4_modem, "THROB4", "Throb 4", "", "THRB", "TB4", DISABLED_IO },
-	{ MODE_THROBX1, &throbx1_modem, "THRBX1", "ThrobX 1", "", "THRBX", "TX1", DISABLED_IO },
-	{ MODE_THROBX2, &throbx2_modem, "THRBX2", "ThrobX 2", "", "THRBX", "TX2", DISABLED_IO },
-	{ MODE_THROBX4, &throbx4_modem, "THRBX4", "ThrobX 4", "", "THRBX", "TX4", DISABLED_IO },
+	{ MODE_THROB1, &throb1_modem, "THROB1", "Throb 1", "", "THROB1", "TB1", DISABLED_IO },
+	{ MODE_THROB2, &throb2_modem, "THROB2", "Throb 2", "", "THROB2", "TB2", DISABLED_IO },
+	{ MODE_THROB4, &throb4_modem, "THROB4", "Throb 4", "", "THROB4", "TB4", DISABLED_IO },
+	{ MODE_THROBX1, &throbx1_modem, "THRBX1", "ThrobX 1", "", "THRBX1", "TX1", DISABLED_IO },
+	{ MODE_THROBX2, &throbx2_modem, "THRBX2", "ThrobX 2", "", "THRBX2", "TX2", DISABLED_IO },
+	{ MODE_THROBX4, &throbx4_modem, "THRBX4", "ThrobX 4", "", "THRBX4", "TX4", DISABLED_IO },
 
 //	{ MODE_PACKET, &pkt_modem, "PACKET", "Packet", "", "PKT", "PKT", ARQ_IO | KISS_IO },
 
@@ -200,8 +200,7 @@ const struct mode_info_t mode_info[NUM_MODES] = {
 
 	{ MODE_SSB, &ssb_modem, "SSB", "SSB", "", "SSB", "", DISABLED_IO },
 	{ MODE_WWV, &wwv_modem, "WWV", "WWV", "", "", "", DISABLED_IO },
-	{ MODE_ANALYSIS, &anal_modem, "ANALYSIS", "Freq Analysis", "", "", "", DISABLED_IO },
-	{ MODE_FFTSCAN, &fftscan_modem, "FREQSCAN", "Freq Scan", "", "", "", DISABLED_IO }
+	{ MODE_ANALYSIS, &anal_modem, "ANALYSIS", "Freq Analysis", "", "", "", DISABLED_IO }
 };
 
 std::ostream& operator<<(std::ostream& s, const qrg_mode_t& m)
