@@ -66,10 +66,12 @@ public:
 protected:
 	enum {
 		RX_MENU_QRZ_THIS, RX_MENU_CALL, RX_MENU_NAME, RX_MENU_QTH,
-		RX_MENU_STATE, RX_MENU_PROVINCE,RX_MENU_COUNTRY, RX_MENU_LOC,
+		RX_MENU_STATE, RX_MENU_COUNTY, RX_MENU_PROVINCE,
+		RX_MENU_COUNTRY, RX_MENU_LOC,
 		RX_MENU_RST_IN,
 		RX_MENU_XCHG, RX_MENU_SERIAL,
 		RX_MENU_CLASS, RX_MENU_SECTION,
+		RX_MENU_SS_SER, RX_MENU_SS_PRE, RX_MENU_SS_CHK, RX_MENU_SS_SEC,
 		RX_MENU_CQZONE, RX_MENU_CQSTATE,
 		RX_MENU_DIV, RX_MENU_COPY, RX_MENU_CLEAR, RX_MENU_SELECT_ALL,
 		RX_MENU_SAVE, RX_MENU_WRAP, RX_MENU_ALL_ENTRY, RX_MENU_SCROLL_HINTS
