@@ -106,6 +106,7 @@ extern Fl_Input2		*inpRstOut;
 extern Fl_Input2		*inpQth;
 extern Fl_Input2		*inpState;
 extern Fl_Input2		*inpCountry;
+extern Fl_Input2		*inpCounty;
 extern Fl_Input2		*inpSerNo;
 extern Fl_Input2		*outSerNo;
 extern Fl_Input2		*outSerNo1;
@@ -115,6 +116,8 @@ extern Fl_Input2		*inpXchgIn1;
 extern Fl_Input2		*inpXchgIn2;
 extern Fl_Input2		*inpVEprov;
 extern Fl_Input2		*inpLoc;
+extern Fl_Input2		*inpLoc1;
+extern Fl_Input2		*inpLoc2;
 extern Fl_Input2		*inpNotes;
 extern Fl_Input2		*inpAZ;	// WA5ZNU
 extern Fl_Button		*qsoSave;
@@ -291,7 +294,6 @@ extern Fl_Light_Button		*btnTune;
 extern Fl_Light_Button		*btnAutoSpot;
 
 extern bool			bWF_only;
-extern bool			withnoise;
 extern int				altMacros;
 
 extern waterfall		*wf;
@@ -339,7 +341,6 @@ extern void set_phase(double phase, double quality, bool highlight);
 extern void set_rtty(double, double, double);
 extern void set_video(double *, int, bool = true);
 extern void set_zdata(cmplx *, int);
-extern void update_scope();
 
 extern void set_scope_xaxis_1(double y1);
 extern void set_scope_xaxis_2(double y2);
@@ -393,6 +394,7 @@ extern void rxid_selection_color();
 extern void txid_selection_color();
 extern void tune_selection_color();
 extern void spot_selection_color();
+extern void set_default_btn_color();
 
 extern void put_WARNstatus(double);
 
@@ -502,6 +504,7 @@ extern bool xml_get_record(const char *);
 extern const char* zdate(void);
 extern const char* ztime(void);
 extern void ztimer(void* first_call);
+extern const unsigned long zmsec(void);
 
 extern void setTabColors();
 
@@ -522,6 +525,8 @@ extern void set_rtty_shift(int shift);
 extern void set_rtty_baud(float baud);
 extern void set_rtty_bits(int bits);
 extern void set_rtty_bw(float bw);
+extern void enable_rtty_quickchange();
+extern void disable_rtty_quickchange();
 
 extern void sync_cw_parameters();
 
@@ -597,11 +602,20 @@ extern Fl_Input2	*inp_FD_section2;
 extern Fl_Input2	*inp_CQzone;
 extern Fl_Input2	*inp_CQstate;
 
+// CW Sweepstakes
+extern Fl_Input2	*inp_SS_SerialNoR;
+extern Fl_Input2	*inp_SS_Precedence;
+extern Fl_Input2	*inp_SS_Check;
+extern Fl_Input2	*inp_SS_Section;
+
 extern Fl_Double_Window	*dxcluster_viewer;
 
 // TOD clock
 extern void  TOD_init();
 extern void  TOD_close();
+extern void  start_deadman();
+extern void  stop_deadman();
+extern void  disable_deadman();
 
 //======================================================================
 //FeldHell resizable Rx character height
