@@ -84,14 +84,22 @@ void loadBrowser(Fl_Widget *widget) {
 
 	w->add(LINE_SEP);
 	w->add(_("<CALL>\tother call"));
-	w->add(_("<INFO1>\tS/N etc."));
-	w->add(_("<INFO2>\tIMD etc."));
-	w->add(_("<LOC>\tother locator"));
 	w->add(_("<NAME>\tother name"));
 	w->add(_("<QTH>\tother QTH"));
+	w->add(_("<ST>\tother State"));
+	w->add(_("<PR>\tother Province"));
+	w->add(_("<LOC>\tother locator"));
 	w->add(_("<RST>\tother RST"));
+
+	w->add(LINE_SEP);
+	w->add(_("<INFO1>\tS/N etc."));
+	w->add(_("<INFO2>\tIMD etc."));
+
+	w->add(LINE_SEP);
 	w->add(_("<QSONBR>\t# QSO recs"));
 	w->add(_("<NXTNBR>\tnext QSO rec #"));
+
+	w->add(LINE_SEP);
 	w->add(_("<MAPIT>\tmap on google"));
 	w->add(_("<MAPIT:adr/lat/loc>\tmap by value"));
 
@@ -140,6 +148,8 @@ void loadBrowser(Fl_Widget *widget) {
 	w->add(_("<SAVEXCHG>\tsave contest out"));
 	w->add(_("<SERNO>\tcurrent contest serno"));
 	w->add(_("<LASTNO>\tlast serno sent"));
+	w->add(_("<FDCLASS>\trecvd FD class"));
+	w->add(_("<FDSECT>\trecvd FD section"));
 
 	w->add(LINE_SEP);
 	w->add(_("<RX>\treceive"));
@@ -165,6 +175,7 @@ void loadBrowser(Fl_Widget *widget) {
 	w->add(LINE_SEP);
 	w->add(_("<FILE:>\tinsert text file"));
 	w->add(_("<IMAGE:>\tinsert image file"));
+	w->add(_("<AVATAR>\tsend avatar"));
 	w->add(LINE_SEP);
 
 	w->add(_("<PAUSE>\tpause transmit"));
@@ -188,6 +199,10 @@ void loadBrowser(Fl_Widget *widget) {
 	w->add(_("<NRSID:NN>\tTransmit |NN| successive RsID bursts"));
 	w->add(_("<DTMF:[Wn:][Ln:]chrs>\t[Wait][Len](ms)"));
 
+	w->add(LINE_SEP);
+	w->add(_("<ALERT:[bark][checkout][doesnot][phone][beeboo][diesel][steam_train][dinner_bell]>"));
+	w->add(_("<ALERT:>\talert using external wav file"));
+
 	w->add(LINE_SEP);
 	w->add(_("<POST:+/-nn.n>\tCW QSK post-timing"));
 	w->add(_("<PRE:nn.n>\tCW QSK pre-timing"));
@@ -196,6 +211,7 @@ void loadBrowser(Fl_Widget *widget) {
 
 	w->add(LINE_SEP);
 	w->add(_("<RIGCAT:[\"text\"][hex ...]:ret>\tsend CAT cmd"));
+	w->add(_("<FLRIG:[\"text\"][hex ...]>\tsend CAT cmd"));
 
 	w->add(LINE_SEP);
 	w->add(_("<AFC:on|off|t>\tAFC  on,off,toggle"));
@@ -386,6 +402,16 @@ void cbInsertMacro(Fl_Widget *, void *)
 			text.insert(8, p);
 		} else
 			text = "";
+	} else if (text == "<ALERT:>") {
+		string filters = "Wav file\t*.wav";
+		const char* p = FSEL::select(
+			_("Select wav file"),
+			filters.c_str(),
+			HomeDir.c_str());
+		if (p && *p) {
+			text.insert(7, p);
+		} else
+			text = "";
 	}
 #ifdef __MINGW32__
 	else if (text == "<EXEC>") {
