@@ -165,35 +165,63 @@ SoundBase::~SoundBase()
 }
 
 #if USE_SNDFILE
-void SoundBase::get_file_params(const char* def_fname, const char** fname, int* format)
+void SoundBase::get_file_params(std::string def_fname, std::string &fname, int &format)
 {
-	std::string filters = _("Waveform Audio Format\t*.wav\n" "AU\t*.{au,snd}\n");
-	int nfilt = 2;
+	std::string filters = _("Waveform Audio Format\t*.wav\n");
+	filters.append(_("AU\t*.{au,snd}\n"));
 	if (format_supported(SF_FORMAT_FLAC | SF_FORMAT_PCM_16)) {
-		filters += _("Free Lossless Audio Codec\t*.flac");
-		nfilt++;
+		filters.append(_("Free Lossless Audio Codec\t*.flac"));
 	}
 
+	format = SF_FORMAT_WAV | SF_FORMAT_PCM_16;
 	int fsel;
-	if (strstr(def_fname, "playback"))
-		*fname = FSEL::select(_("Audio file"), filters.c_str(), def_fname, &fsel);
+	const char *fn = 0;
+	if (def_fname.find("playback") != std::string::npos)
+		fn = FSEL::select(_("Audio file"), filters.c_str(), def_fname.c_str(), &fsel);
 	else
-		*fname = FSEL::saveas(_("Audio file"), filters.c_str(), def_fname, &fsel);
-	if (!*fname) return;
-	if (!**fname) return;
+		fn = FSEL::saveas(_("Audio file"), filters.c_str(), def_fname.c_str(), &fsel);
 
-	if (fsel >= nfilt) // "Default" save-as type on OS X
-		fsel = 0;
+	if (!fn || !*fn) {
+		fname = "";
+		return;
+	}
+	fname = fn;
+
+	bool check_replace = false;
+	fsel = 0;
 	switch (fsel) {
-	case 0:
-		*format = SF_FORMAT_WAV | SF_FORMAT_PCM_16;
-		break;
-	case 1:
-		*format = SF_FORMAT_AU | SF_FORMAT_FLOAT | SF_ENDIAN_CPU;
-		break;
-	case 2:
-		*format = SF_FORMAT_FLAC | SF_FORMAT_PCM_16;
-		break;
+		default:
+		case 0:
+			format = SF_FORMAT_WAV | SF_FORMAT_PCM_16;
+			if (fname.find(".wav") == std::string::npos) {
+				fname.append(".wav");
+				check_replace = true;
+			}
+			break;
+		case 1:
+			format = SF_FORMAT_AU | SF_FORMAT_FLOAT | SF_ENDIAN_CPU;
+			if (fname.find(".au") == std::string::npos ||
+				fname.find(".snd") == std::string::npos) {
+				fname.append(".au");
+				check_replace = true;
+			}
+			break;
+		case 2:
+			format = SF_FORMAT_FLAC | SF_FORMAT_PCM_16;
+			if (fname.find(".flac") == std::string::npos) {
+				fname.append(".flac");
+				check_replace = true;
+			}
+			break;
+	}
+
+	if (check_replace) {
+		FILE *f = fopen(fname.c_str(), "r");
+		if (f) {
+			fclose(f);
+			int ans = fl_choice("Replace %s?", "Yes", "No", 0, fname.c_str());
+			if ( ans == 1) fname = "";
+		}
 	}
 }
 
@@ -210,10 +238,11 @@ int SoundBase::Capture(bool val)
 		return 1;
 	}
 
-	const char* fname;
+	std::string fname;
 	int format;
-	get_file_params("capture.wav", &fname, &format);
-	if (!fname)
+	get_file_params("capture", fname, format);
+
+	if (fname.empty())
 		return 0;
 
 	// frames (ignored), freq, channels, format, sections (ignored), seekable (ignored)
@@ -221,8 +250,8 @@ int SoundBase::Capture(bool val)
 		progdefaults.record_both_channels ? 2 : 1,
 //		SNDFILE_CHANNELS,
 		format, 0, 0 };
-	if ((ofCapture = sf_open(fname, SFM_WRITE, &info)) == NULL) {
-		LOG_ERROR("Could not write %s:%s", fname, sf_strerror(NULL) );
+	if ((ofCapture = sf_open(fname.c_str(), SFM_WRITE, &info)) == NULL) {
+		LOG_ERROR("Could not write %s:%s", fname.c_str(), sf_strerror(NULL) );
 		return 0;
 	}
 	if (sf_command(ofCapture, SFC_SET_UPDATE_HEADER_AUTO, NULL, SF_TRUE) != SF_TRUE)
@@ -246,18 +275,18 @@ int SoundBase::Generate(bool val)
 		return 1;
 	}
 
-	const char* fname;
+	std::string fname;
 	int format;
-	get_file_params("generate.wav", &fname, &format);
-	if (!fname)
+	get_file_params("generate", fname, format);
+	if (fname.empty())
 		return 0;
 
 	SF_INFO info = { 0, sndfile_samplerate[progdefaults.wavSampleRate], 
 		progdefaults.record_both_channels ? 2 : 1,
 //		SNDFILE_CHANNELS,
 		format, 0, 0 };
-	if ((ofGenerate = sf_open(fname, SFM_WRITE, &info)) == NULL) {
-		LOG_ERROR("Could not write %s", fname);
+	if ((ofGenerate = sf_open(fname.c_str(), SFM_WRITE, &info)) == NULL) {
+		LOG_ERROR("Could not write %s", fname.c_str());
 		return 0;
 	}
 	if (sf_command(ofGenerate, SFC_SET_UPDATE_HEADER_AUTO, NULL, SF_TRUE) != SF_TRUE)
@@ -290,10 +319,10 @@ int SoundBase::Playback(bool val)
 		playback = false;
 		return 1;
 	}
-	const char* fname;
+	std::string fname;
 	int format;
-	get_file_params("playback.wav", &fname, &format);
-	if (!fname)
+	get_file_params("playback", fname, format);
+	if (fname.empty())
 		return -1;
 
 	play_info.frames = 0;
@@ -303,11 +332,12 @@ int SoundBase::Playback(bool val)
 	play_info.sections = 0;
 	play_info.seekable = 0;
 
-	if ((ifPlayback = sf_open(fname, SFM_READ, &play_info)) == NULL) {
-		LOG_ERROR("Could not read %s:%s", fname, sf_strerror(NULL) );
+	if ((ifPlayback = sf_open(fname.c_str(), SFM_READ, &play_info)) == NULL) {
+		LOG_ERROR("Could not read %s:%s", fname.c_str(), sf_strerror(NULL) );
 		return -2;
 	}
-LOG_VERBOSE("wav file stats:\n\
+LOG_VERBOSE
+("wav file stats:\n\
 frames     : %d\n\
 samplerate : %d\n\
 channels   : %d\n\
@@ -321,16 +351,11 @@ play_info.format,
 play_info.sections,
 play_info.seekable);
 
-// this restriction needs to be removed!
-//	if (play_info.channels != 1) {
-//		sf_close(ifPlayback);
-//		return -3;
-//	}
-
 	modem_play_sr = sample_frequency;
 	play_src_data->src_ratio = 1.0 * modem_play_sr / play_info.samplerate;
 	src_set_ratio(play_src_state, play_src_data->src_ratio);
-LOG_VERBOSE("src ratio %f", play_src_data->src_ratio);
+LOG_VERBOSE
+("src ratio %f", play_src_data->src_ratio);
 
 	progdefaults.loop_playback = fl_choice2(_("Playback continuous loop?"), _("No"), _("Yes"), NULL);
 
@@ -501,16 +526,12 @@ bool SoundBase::format_supported(int format)
 		0,
 		sndfile_samplerate[progdefaults.wavSampleRate],
 		progdefaults.record_both_channels ? 2 : 1,
-//		SNDFILE_CHANNELS,
 		format, 0, 0 };
-	FILE* f;
-	if ((f = tmpfile()) == NULL)
-		return false;
-	SNDFILE* sndf = sf_open_fd(fileno(f), SFM_WRITE, &info, SF_FALSE);
-	fclose(f);
-	if (sndf)
-		sf_close(sndf);
-	return sndf;
+	SNDFILE* sndf = sf_open("temp.audio", SFM_WRITE, &info);
+	sf_close(sndf);
+	remove("temp.audio");
+	if (sndf) return true;
+	return false;
 }
 
 void SoundBase::tag_file(SNDFILE *sndfile, const char *title)
@@ -633,6 +654,7 @@ int SoundOSS::Open(int md, int freq)
 		device_fd = fl_open(device.c_str(), oflags, 0);
 		if (device_fd == -1)
 			throw SndException(errno);
+
 		Format(AFMT_S16_LE);	// default: 16 bit little endian
 		Channels(2);			//		2 channels
 		Frequency(freq);
@@ -1052,7 +1074,8 @@ SoundPort::SoundPort(const char *in_dev, const char *out_dev) : req_sample_rate(
 #endif
 	for (size_t i = 0; i < sizeof(sems)/sizeof(*sems); i++) {
 #if USE_NAMED_SEMAPHORES
-		snprintf(sname, sizeof(sname), "%" PRIuSZ "-%u-%s", i, getpid(), PACKAGE_TARNAME);
+		unsigned int un = i;
+		snprintf(sname, sizeof(sname), "%u-%u-%s", un, getpid(), PACKAGE_TARNAME);
 		if ((*sems[i] = sem_open(sname, O_CREAT | O_EXCL, 0600, 0)) == (sem_t*)SEM_FAILED) {
 			pa_perror(errno, sname);
 			throw SndException(errno);
@@ -1139,7 +1162,7 @@ int SoundPort::Open(int mode, int freq)
 	req_sample_rate = sample_frequency = freq;
 
 	// do we need to (re)initialise the streams?
-	int ret = 0;
+	int ret = 1;
 	int sr[2] = { progdefaults.in_sample_rate, progdefaults.out_sample_rate };
 
 	// initialize stream if it is a JACK device, regardless of mode
@@ -1165,12 +1188,12 @@ int SoundPort::Open(int mode, int freq)
 			// reset the semaphore
 			while (sem_trywait(sd[i].rwsem) == 0);
 			if (errno && errno != EAGAIN) {
-				pa_perror(errno, "open");
+				pa_perror(errno, "open failed");
 				throw SndException(errno);
 			}
 			start_stream(i);
 
-			ret = 1;
+			ret = 0;
 		}
 		else {
 			pause_stream(i);
@@ -1561,23 +1584,23 @@ void SoundPort::src_data_reset(unsigned dir)
 	if (dir == 0) {
 		if (rx_src_state)
 			src_delete(rx_src_state);
-			rx_src_state = src_callback_new(src_read_cb, progdefaults.sample_converter,
-											sd[0].params.channelCount, &err, &sd[0]);
-			if (!rx_src_state) {
-				pa_perror(err, src_strerror(err));
-				throw SndException(src_strerror(err));
-			}
-			sd[0].src_ratio = req_sample_rate / (sd[0].dev_sample_rate * (1.0 + rxppm / 1e6));
+		rx_src_state = src_callback_new(src_read_cb, progdefaults.sample_converter,
+										sd[0].params.channelCount, &err, &sd[0]);
+		if (!rx_src_state) {
+			pa_perror(err, src_strerror(err));
+			throw SndException(src_strerror(err));
+		}
+		sd[0].src_ratio = req_sample_rate / (sd[0].dev_sample_rate * (1.0 + rxppm / 1e6));
 	}
 	else if (dir == 1) {
 		if (tx_src_state)
 			src_delete(tx_src_state);
-			tx_src_state = src_new(progdefaults.sample_converter, sd[1].params.channelCount, &err);
-			if (!tx_src_state) {
-				pa_perror(err, src_strerror(err));
-				throw SndException(src_strerror(err));
-			}
-			tx_src_data->src_ratio = sd[1].dev_sample_rate * (1.0 + txppm / 1e6) / req_sample_rate;
+		tx_src_state = src_new(progdefaults.sample_converter, sd[1].params.channelCount, &err);
+		if (!tx_src_state) {
+			pa_perror(err, src_strerror(err));
+			throw SndException(src_strerror(err));
+		}
+		tx_src_data->src_ratio = sd[1].dev_sample_rate * (1.0 + txppm / 1e6) / req_sample_rate;
 	}
 
 	rbsize = 2 * MAX(ceil2(
@@ -1585,7 +1608,9 @@ void SoundPort::src_data_reset(unsigned dir)
 					MAX(req_sample_rate, sd[dir].dev_sample_rate) /
 					MIN(req_sample_rate, sd[dir].dev_sample_rate))),
 					8192);
-	LOG_VERBOSE("rbsize = %" PRIuSZ "", rbsize);
+	stringstream info;
+	info << "rbsize = " << rbsize;
+	LOG_VERBOSE("%s", info.str().c_str());
 	if (sd[dir].rb) delete sd[dir].rb;
 	sd[dir].rb = new ringbuffer<float>(rbsize);
 }
@@ -1636,7 +1661,7 @@ void SoundPort::init_stream(unsigned dir)
 
 	if ((sd[dir].idev = name_to_device(sd[dir].device, dir)) != devs.end())
 		idx = sd[dir].idev - devs.begin();
-		if (idx == paNoDevice) { // no match
+	if (idx == paNoDevice) { // no match
 		LOG_ERROR("Could not find device \"%s\", using default device", sd[dir].device.c_str());
 		PaDeviceIndex def = (dir == 0 ? Pa_GetDefaultInputDevice() : Pa_GetDefaultOutputDevice());
 		if (def == paNoDevice) {
@@ -1645,7 +1670,7 @@ void SoundPort::init_stream(unsigned dir)
 		}
 		sd[dir].idev = devs.begin() + def;
 		idx = def;
-		}
+	}
 	else if (sd[dir].idev == devs.end()) // if we only found a near-match point the idev iterator to it
 		sd[dir].idev = devs.begin() + idx;
 
@@ -1744,7 +1769,7 @@ void SoundPort::start_stream(unsigned dir)
 			paNoFlag,
 			stream_process, &sd[dir]);
 	if (err != paNoError) {
-		pa_perror(err, "Portaudio open stream error");
+//		pa_perror(err, "Portaudio open stream error");
 		throw SndPortException(err);
 	}
 
@@ -2000,7 +2025,7 @@ SoundPulse::~SoundPulse()
 	delete [] rbuf;
 }
 
-int SoundPulse::Open(int mode, int freq)
+int SoundPulse::Open(int dir, int freq)
 {
 	const char* server = (progdefaults.PulseServer.length() ?
 				  progdefaults.PulseServer.c_str() : NULL);
@@ -2008,23 +2033,19 @@ int SoundPulse::Open(int mode, int freq)
 	int err;
 
 	sample_frequency = freq;
-	for (int i = 0; i < 2; i++) {
-		src_data_reset(1 << O_RDONLY | 1 << O_WRONLY);
 
-		if ((unsigned)freq != sd[i].stream_params.rate)
-			Close(i);
-		if (sd[i].stream)
-			continue;
+	src_data_reset(1 << O_RDONLY | 1 << O_WRONLY);
+	if ((unsigned)freq != sd[dir].stream_params.rate)
+		Close(dir);
 
-		sd[i].stream_params.rate = freq;
-		snprintf(sname, sizeof(sname), "%s (%u)", (i ? "playback" : "capture"), getpid());
-		setenv("PULSE_PROP_application.icon_name", PACKAGE_TARNAME, 1);
-		sd[i].stream = pa_simple_new(server, main_window_title.c_str(), sd[i].dir, NULL,
-						 sname, &sd[i].stream_params, NULL,
-						 &sd[i].buffer_attrs, &err);
-		if (!sd[i].stream)
-			throw SndPulseException(err);
-	}
+	sd[dir].stream_params.rate = freq;
+	snprintf(sname, sizeof(sname), "%s (%u)", (dir ? "playback" : "capture"), getpid());
+	setenv("PULSE_PROP_application.icon_name", PACKAGE_TARNAME, 1);
+	sd[dir].stream = pa_simple_new(server, main_window_title.c_str(), sd[dir].dir, NULL,
+				 sname, &sd[dir].stream_params, NULL,
+				 &sd[dir].buffer_attrs, &err);
+	if (!sd[dir].stream)
+		throw SndPulseException(err);
 
 	return 0;
 }
