@@ -105,6 +105,7 @@ private:
 	RX_MENU_NAME
 	RX_MENU_QTH,
 	RX_MENU_STATE
+	RX_MENU_COUNTY,
 	RX_MENU_PROVINCE
 	RX_MENU_COUNTRY
 	RX_MENU_LOC,
@@ -113,6 +114,10 @@ private:
 	RX_MENU_SERIAL
 	RX_MENU_CLASS
 	RX_MENU_SECTION,
+	RX_MENU_SS_SER,
+	RX_MENU_SS_PRE,
+	RX_MENU_SS_CHK,
+	RX_MENU_SS_SEC,
 	RX_MENU_CQZONE,
 	RX_MENU_CQSTATE,
 	RX_MENU_DIV
@@ -131,14 +136,19 @@ Fl_Menu_Item FTextRX::menu[] = {
 	{ icons::make_icon_label(_("Name"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
 	{ icons::make_icon_label(_("QTH"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
 	{ icons::make_icon_label(_("State"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
-	{ icons::make_icon_label(_("Province"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ icons::make_icon_label(_("County"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ icons::make_icon_label(_("Province"), enter_key_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
 	{ icons::make_icon_label(_("Country"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
 	{ icons::make_icon_label(_("Locator"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
-	{ icons::make_icon_label(_("RST(r)"), enter_key_icon), 0, 0, 0,  FL_MENU_DIVIDER, _FL_MULTI_LABEL },
+	{ icons::make_icon_label(_("RST(r)"), enter_key_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
 	{ icons::make_icon_label(_("Exchange In"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
 	{ icons::make_icon_label(_("Serial number"), enter_key_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
 	{ icons::make_icon_label(_("FD class"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
 	{ icons::make_icon_label(_("FD section"), enter_key_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
+	{ icons::make_icon_label(_("SS ser #"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ icons::make_icon_label(_("SS prec"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ icons::make_icon_label(_("SS check"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
+	{ icons::make_icon_label(_("SS section"), enter_key_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
 	{ icons::make_icon_label(_("CQ zone"), enter_key_icon), 0, 0, 0, 0, _FL_MULTI_LABEL },
 	{ icons::make_icon_label(_("CQ STATE"), enter_key_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
 	{ icons::make_icon_label(_("Insert marker"), insert_link_icon), 0, 0, 0, FL_MENU_DIVIDER, _FL_MULTI_LABEL },
@@ -248,6 +258,8 @@ int FTextRX::handle(int event)
 	// catch some text-modifying events that are not handled by kf_* functions
 	case FL_KEYBOARD:
 		break;
+	case FL_PASTE:
+		return 0;
 	case FL_ENTER:
 		if (!progdefaults.rxtext_tooltips || Fl_Tooltip::delay() == 0.0f)
 			break;
@@ -472,12 +484,26 @@ int FTextRX::handle_qso_data(int start, int end)
 	Fl_Input2* target = 0;
 
 	if (progdefaults.logging != LOG_QSO) {
-		if (call.match(s)) { // point p to substring
+		if (loc.match(s)) { // force maidenhead match to exchange
+							// or it will overwrite the call
+			inpXchgIn->position(inpXchgIn->size());
+			if (inpXchgIn->size()) inpXchgIn->insert(" ", 1);
+			inpXchgIn->insert(s);
+			log_callback(inpXchgIn);
+		} else if (call.match(s)) { // point p to substring
 			const regmatch_t& offsets = call.suboff()[1];
 			p = s + offsets.rm_so;
 			*(s + offsets.rm_eo) = '\0';
 			inpCall->value(p);
-			log_callback(inpCall);
+			if (progdefaults.clear_fields) {
+				inpXchgIn->value("");
+				inpName->value("");
+				inp_FD_class->value("");
+				inp_FD_section->value("");
+				inp_CQzone->value("");
+				inp_CQstate->value("");
+				log_callback(inpCall);
+			}
 			Fl::copy(p, strlen(p), 1);  // copy to clipboard
 		} else if (progdefaults.logging == LOG_FD) { 
 			if (inp_FD_class->value()[0] == 0)
@@ -525,11 +551,7 @@ int FTextRX::handle_qso_data(int start, int end)
 
 void FTextRX::handle_context_menu(void)
 {
-	bool contest_ui =
-			progStatus.Rig_Contest_UI || 
-			(progdefaults.logging != LOG_QSO) ||
-			progStatus.contest;
-	if (progStatus.NO_RIGLOG || progStatus.Rig_Log_UI) contest_ui = false;
+	bool contest_ui = (progdefaults.logging != LOG_QSO);
 
 	unsigned shown = 0; // all hidden
 
@@ -546,10 +568,15 @@ void FTextRX::handle_context_menu(void)
 		} else if (progdefaults.logging == LOG_CQWW) {
 			show_item(RX_MENU_CQZONE);
 			show_item(RX_MENU_CQSTATE);
-		} else if (	progdefaults.logging == LOG_CONT || 
+		} else if ( progdefaults.logging == LOG_CONT || 
 					progdefaults.logging == LOG_BART) {
 			show_item(RX_MENU_SERIAL);
 			show_item(RX_MENU_XCHG);
+		} else if ( progdefaults.logging == LOG_CWSS) {
+			show_item(RX_MENU_SS_SER);
+			show_item(RX_MENU_SS_PRE);
+			show_item(RX_MENU_SS_CHK);
+			show_item(RX_MENU_SS_SEC);
 		}
 	}
 	else {
@@ -633,6 +660,9 @@ void FTextRX::menu_cb(size_t item)
 	case RX_MENU_LOC:
 		input = inpLoc;
 		break;
+	case RX_MENU_COUNTY:
+		input = inpCounty;
+		break;
 	case RX_MENU_RST_IN:
 		input = inpRstIn;
 		break;
@@ -648,6 +678,18 @@ void FTextRX::menu_cb(size_t item)
 	case RX_MENU_SECTION:
 		input = inp_FD_section;
 		break;
+	case RX_MENU_SS_SER:
+		input = inp_SS_SerialNoR;
+		break;
+	case RX_MENU_SS_PRE:
+		input = inp_SS_Precedence;
+		break;
+	case RX_MENU_SS_CHK:
+		input = inp_SS_Check;
+		break;
+	case RX_MENU_SS_SEC:
+		input = inp_SS_Section;
+		break;
 	case RX_MENU_CQZONE:
 		input = inp_CQzone;
 		break;
@@ -691,7 +733,7 @@ void FTextRX::menu_cb(size_t item)
 
 	if (!input)
 		return;
-	char* s = get_word(popx, popy, progdefaults.nonwordchars.c_str());
+	char* s = get_word(popx, popy, progdefaults.nonwordchars.c_str(), false);
 	if (!s)
 		return;
 
